"""FortiOS CMDB - System category"""

from . import admin
from . import api_user
from . import automation_action
from . import automation_stitch
from . import available_interfaces
from . import botnet
from . import botnet_domains
from . import central_management
from . import certificate
from . import change_password
from . import cluster
from . import com_log
from . import config
from . import config_error_log
from . import config_revision
from . import config_script
from . import config_sync
from . import crash_log
from . import csf
from . import debug
from . import dhcp
from . import dhcp6
from . import disconnect_admins
from . import external_resource
from . import firmware
from . import fortiguard
from . import fortimanager
from . import fsck
from . import ha_peer
from . import hscalefw_license
from . import interface
from . import ipam
from . import logdisk
from . import lte_modem
from . import modem
from . import modem5g
from . import ntp
from . import object
from . import os
from . import password_policy_conform
from . import performance
from . import private_data_encryption
from . import process
from . import resource
from . import sandbox
from . import sdn_connector
from . import time
from . import traffic_history
from . import upgrade_report
from . import usb_device
from . import usb_log
from . import vmlicense
from .acme_certificate_status import AcmeCertificateStatus
from .acquired_dns import AcquiredDns
from .available_certificates import AvailableCertificates
from .check_port_availability import CheckPortAvailability
from .current_admins import CurrentAdmins
from .global_resources import GlobalResources
from .global_search import GlobalSearch
from .ha_backup_hb_used import HaBackupHbUsed
from .ha_checksums import HaChecksums
from .ha_history import HaHistory
from .ha_hw_interface import HaHwInterface
from .ha_nonsync_checksums import HaNonsyncChecksums
from .ha_statistics import HaStatistics
from .ha_table_checksums import HaTableChecksums
from .interface_connected_admins_info import InterfaceConnectedAdminsInfo
from .ipconf import Ipconf
from .link_monitor import LinkMonitor
from .modem3g import Modem3g
from .monitor_sensor import MonitorSensor
from .resolve_fqdn import ResolveFqdn
from .running_processes import RunningProcesses
from .sensor_info import SensorInfo
from .status import Status
from .storage import Storage
from .timezone import Timezone
from .trusted_cert_authorities import TrustedCertAuthorities
from .vdom_link import VdomLink
from .vdom_resource import VdomResource
from .vm_information import VmInformation

__all__ = [
    "AcmeCertificateStatus",
    "AcquiredDns",
    "Admin",
    "ApiUser",
    "AutomationAction",
    "AutomationStitch",
    "AvailableCertificates",
    "AvailableInterfaces",
    "Botnet",
    "BotnetDomains",
    "CentralManagement",
    "Certificate",
    "ChangePassword",
    "CheckPortAvailability",
    "Cluster",
    "ComLog",
    "Config",
    "ConfigErrorLog",
    "ConfigRevision",
    "ConfigScript",
    "ConfigSync",
    "CrashLog",
    "Csf",
    "CurrentAdmins",
    "Debug",
    "Dhcp",
    "Dhcp6",
    "DisconnectAdmins",
    "ExternalResource",
    "Firmware",
    "Fortiguard",
    "Fortimanager",
    "Fsck",
    "GlobalResources",
    "GlobalSearch",
    "HaBackupHbUsed",
    "HaChecksums",
    "HaHistory",
    "HaHwInterface",
    "HaNonsyncChecksums",
    "HaPeer",
    "HaStatistics",
    "HaTableChecksums",
    "HscalefwLicense",
    "Interface",
    "InterfaceConnectedAdminsInfo",
    "Ipam",
    "Ipconf",
    "LinkMonitor",
    "Logdisk",
    "LteModem",
    "Modem",
    "Modem3g",
    "Modem5g",
    "MonitorSensor",
    "Ntp",
    "Object",
    "Os",
    "PasswordPolicyConform",
    "Performance",
    "PrivateDataEncryption",
    "Process",
    "ResolveFqdn",
    "Resource",
    "RunningProcesses",
    "Sandbox",
    "SdnConnector",
    "SensorInfo",
    "Status",
    "Storage",
    "System",
    "Time",
    "Timezone",
    "TrafficHistory",
    "TrustedCertAuthorities",
    "UpgradeReport",
    "UsbDevice",
    "UsbLog",
    "VdomLink",
    "VdomResource",
    "VmInformation",
    "Vmlicense",
]


class System:
    """System endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """System endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.admin = admin.Admin(client)
        self.api_user = api_user.ApiUser(client)
        self.automation_action = automation_action.AutomationAction(client)
        self.automation_stitch = automation_stitch.AutomationStitch(client)
        self.available_interfaces = available_interfaces.AvailableInterfaces(client)
        self.botnet = botnet.Botnet(client)
        self.botnet_domains = botnet_domains.BotnetDomains(client)
        self.central_management = central_management.CentralManagement(client)
        self.certificate = certificate.Certificate(client)
        self.change_password = change_password.ChangePassword(client)
        self.cluster = cluster.Cluster(client)
        self.com_log = com_log.ComLog(client)
        self.config = config.Config(client)
        self.config_error_log = config_error_log.ConfigErrorLog(client)
        self.config_revision = config_revision.ConfigRevision(client)
        self.config_script = config_script.ConfigScript(client)
        self.config_sync = config_sync.ConfigSync(client)
        self.crash_log = crash_log.CrashLog(client)
        self.csf = csf.Csf(client)
        self.debug = debug.Debug(client)
        self.dhcp = dhcp.Dhcp(client)
        self.dhcp6 = dhcp6.Dhcp6(client)
        self.disconnect_admins = disconnect_admins.DisconnectAdmins(client)
        self.external_resource = external_resource.ExternalResource(client)
        self.firmware = firmware.Firmware(client)
        self.fortiguard = fortiguard.Fortiguard(client)
        self.fortimanager = fortimanager.Fortimanager(client)
        self.fsck = fsck.Fsck(client)
        self.ha_peer = ha_peer.HaPeer(client)
        self.hscalefw_license = hscalefw_license.HscalefwLicense(client)
        self.interface = interface.Interface(client)
        self.ipam = ipam.Ipam(client)
        self.logdisk = logdisk.Logdisk(client)
        self.lte_modem = lte_modem.LteModem(client)
        self.modem = modem.Modem(client)
        self.modem5g = modem5g.Modem5g(client)
        self.ntp = ntp.Ntp(client)
        self.object = object.Object(client)
        self.os = os.Os(client)
        self.password_policy_conform = password_policy_conform.PasswordPolicyConform(client)
        self.performance = performance.Performance(client)
        self.private_data_encryption = private_data_encryption.PrivateDataEncryption(client)
        self.process = process.Process(client)
        self.resource = resource.Resource(client)
        self.sandbox = sandbox.Sandbox(client)
        self.sdn_connector = sdn_connector.SdnConnector(client)
        self.time = time.Time(client)
        self.traffic_history = traffic_history.TrafficHistory(client)
        self.upgrade_report = upgrade_report.UpgradeReport(client)
        self.usb_device = usb_device.UsbDevice(client)
        self.usb_log = usb_log.UsbLog(client)
        self.vmlicense = vmlicense.Vmlicense(client)
        self.acme_certificate_status = AcmeCertificateStatus(client)
        self.acquired_dns = AcquiredDns(client)
        self.available_certificates = AvailableCertificates(client)
        self.check_port_availability = CheckPortAvailability(client)
        self.current_admins = CurrentAdmins(client)
        self.global_resources = GlobalResources(client)
        self.global_search = GlobalSearch(client)
        self.ha_backup_hb_used = HaBackupHbUsed(client)
        self.ha_checksums = HaChecksums(client)
        self.ha_history = HaHistory(client)
        self.ha_hw_interface = HaHwInterface(client)
        self.ha_nonsync_checksums = HaNonsyncChecksums(client)
        self.ha_statistics = HaStatistics(client)
        self.ha_table_checksums = HaTableChecksums(client)
        self.interface_connected_admins_info = InterfaceConnectedAdminsInfo(client)
        self.ipconf = Ipconf(client)
        self.link_monitor = LinkMonitor(client)
        self.modem3g = Modem3g(client)
        self.monitor_sensor = MonitorSensor(client)
        self.resolve_fqdn = ResolveFqdn(client)
        self.running_processes = RunningProcesses(client)
        self.sensor_info = SensorInfo(client)
        self.status = Status(client)
        self.storage = Storage(client)
        self.timezone = Timezone(client)
        self.trusted_cert_authorities = TrustedCertAuthorities(client)
        self.vdom_link = VdomLink(client)
        self.vdom_resource = VdomResource(client)
        self.vm_information = VmInformation(client)



"""FortiOS CMDB - Log category"""

from . import av_archive
from . import device
from . import forticloud
from . import forticloud_report
from . import local_report
from . import policy_archive
from . import stats
from .current_disk_usage import CurrentDiskUsage
from .feature_set import FeatureSet
from .fortianalyzer import Fortianalyzer
from .fortianalyzer_queue import FortianalyzerQueue
from .forticloud_report_list import ForticloudReportList
from .historic_daily_remote_logs import HistoricDailyRemoteLogs
from .hourly_disk_usage import HourlyDiskUsage
from .local_report_list import LocalReportList

__all__ = [
    "AvArchive",
    "CurrentDiskUsage",
    "Device",
    "FeatureSet",
    "Fortianalyzer",
    "FortianalyzerQueue",
    "Forticloud",
    "ForticloudReport",
    "ForticloudReportList",
    "HistoricDailyRemoteLogs",
    "HourlyDiskUsage",
    "LocalReport",
    "LocalReportList",
    "Log",
    "PolicyArchive",
    "Stats",
]


class Log:
    """Log endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Log endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.av_archive = av_archive.AvArchive(client)
        self.device = device.Device(client)
        self.forticloud = forticloud.Forticloud(client)
        self.forticloud_report = forticloud_report.ForticloudReport(client)
        self.local_report = local_report.LocalReport(client)
        self.policy_archive = policy_archive.PolicyArchive(client)
        self.stats = stats.Stats(client)
        self.current_disk_usage = CurrentDiskUsage(client)
        self.feature_set = FeatureSet(client)
        self.fortianalyzer = Fortianalyzer(client)
        self.fortianalyzer_queue = FortianalyzerQueue(client)
        self.forticloud_report_list = ForticloudReportList(client)
        self.historic_daily_remote_logs = HistoricDailyRemoteLogs(client)
        self.hourly_disk_usage = HourlyDiskUsage(client)
        self.local_report_list = LocalReportList(client)



"""
FortiOS CMDB - User radius

Configuration endpoint for managing cmdb user/radius objects.

API Endpoints:
    GET    /cmdb/user/radius
    POST   /cmdb/user/radius
    PUT    /cmdb/user/radius/{identifier}
    DELETE /cmdb/user/radius/{identifier}

Example Usage:
    >>> from hfortix_fortios import FortiOS
    >>> fgt = FortiOS(host="192.168.1.99", token="your-api-token")
    >>>
    >>> # List all items
    >>> items = fgt.api.cmdb.user_radius.get()
    >>>
    >>> # Create with auto-normalization (strings/lists converted automatically)
    >>> result = fgt.api.cmdb.user_radius.post(
    ...     name="example",
    ...     srcintf="port1",  # Auto-converted to [{'name': 'port1'}]
    ...     dstintf=["port2", "port3"],  # Auto-converted to list of dicts
    ... )

Important:
    - Use **POST** to create new objects
    - Use **PUT** to update existing objects
    - Use **GET** to retrieve configuration
    - Use **DELETE** to remove objects
    - **Auto-normalization**: List fields accept strings or lists, automatically
      converted to FortiOS format [{'name': '...'}]
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal, Union

if TYPE_CHECKING:
    from collections.abc import Coroutine
    from hfortix_core.http.interface import IHTTPClient
    from hfortix_fortios.models import FortiObject

# Import helper functions from central _helpers module
from hfortix_fortios._helpers import (
    build_api_payload,
    build_cmdb_payload,  # Keep for backward compatibility / manual usage
    is_success,
    quote_path_param,  # URL encoding for path parameters
    normalize_table_field,  # For table field normalization
)
# Import metadata mixin for schema introspection
from hfortix_fortios._helpers.metadata_mixin import MetadataMixin

# Import Protocol-based type hints (eliminates need for local @overload decorators)
from hfortix_fortios._protocols import CRUDEndpoint

class Radius(CRUDEndpoint, MetadataMixin):
    """Radius Operations."""
    
    # Configure metadata mixin to use this endpoint's helper module
    _helper_module_name = "radius"
    
    # ========================================================================
    # Table Fields Metadata (for normalization)
    # Auto-generated from schema - supports flexible input formats
    # ========================================================================
    _TABLE_FIELDS = {
        "class_": {
            "mkey": "name",
            "required_fields": ['name'],
            "example": "[{'name': 'value'}]",
        },
        "accounting_server": {
            "mkey": "id",
            "required_fields": ['server', 'secret', 'interface'],
            "example": "[{'server': '192.168.1.10', 'secret': 'value', 'interface': 'value'}]",
        },
    }
    
    # ========================================================================
    # Capabilities (from schema metadata)
    # ========================================================================
    SUPPORTS_CREATE = True
    SUPPORTS_READ = True
    SUPPORTS_UPDATE = True
    SUPPORTS_DELETE = True
    SUPPORTS_MOVE = True
    SUPPORTS_CLONE = True
    SUPPORTS_FILTERING = True
    SUPPORTS_PAGINATION = True
    SUPPORTS_SEARCH = False
    SUPPORTS_SORTING = False

    def __init__(self, client: "IHTTPClient"):
        """Initialize Radius endpoint."""
        self._client = client

    # ========================================================================
    # GET Method
    # Type hints provided by CRUDEndpoint protocol (no local @overload needed)
    # ========================================================================
    
    def get(
        self,
        name: str | None = None,
        filter: list[str] | None = None,
        count: int | None = None,
        start: int | None = None,
        payload_dict: dict[str, Any] | None = None,
        vdom: str | bool | None = None,
        error_mode: Literal["raise", "return", "print"] | None = None,
        error_format: Literal["detailed", "simple", "code_only"] | None = None,
    ):  # type: ignore[no-untyped-def]
        """
        Retrieve user/radius configuration.

        Configure RADIUS server entries.

        Args:
            name: String identifier to retrieve specific object.
                If None, returns all objects.
            filter: List of filter expressions to limit results.
                Each filter uses format: "field==value" or "field!=value"
                Operators: ==, !=, =@ (contains), !@ (not contains), <=, <, >=, >
                Multiple filters use AND logic. For OR, use comma in single string.
                Example: ["name==test", "status==enable"] or ["name==test,name==prod"]
            count: Maximum number of entries to return (pagination).
            start: Starting entry index for pagination (0-based).
            payload_dict: Additional query parameters for advanced options:
                - datasource (bool): Include datasource information
                - with_meta (bool): Include metadata about each object
                - with_contents_hash (bool): Include checksum of object contents
                - format (list[str]): Property names to include (e.g., ["policyid", "srcintf"])
                - scope (str): Query scope - "global", "vdom", or "both"
                - action (str): Special actions - "schema", "default"
                See FortiOS REST API documentation for complete list.
            vdom: Virtual domain name. Use True for global, string for specific VDOM, None for default.
            error_mode: Override client-level error_mode. "raise" raises exceptions, "return" returns error dict, "print" prints errors.
            error_format: Override client-level error_format. "detailed" provides full context, "simple" is concise, "code_only" returns just status code.

        Returns:
            FortiObject instance or list of FortiObject instances. Returns Coroutine if using async client.
            Use .dict, .json, or .raw properties to access as dictionary.
            
            Response structure:
                - http_method: GET
                - results: Configuration object(s)
                - vdom: Virtual domain
                - path: API path
                - name: Object name (single object queries)
                - status: success/error
                - http_status: HTTP status code
                - build: FortiOS build number

        Examples:
            >>> # Get all user/radius objects
            >>> result = fgt.api.cmdb.user_radius.get()
            >>> print(f"Found {len(result['results'])} objects")
            
            >>> # Get specific user/radius by name
            >>> result = fgt.api.cmdb.user_radius.get(name=1)
            >>> print(result['results'])
            
            >>> # Get with filter
            >>> result = fgt.api.cmdb.user_radius.get(
            ...     filter=["name==test", "status==enable"]
            ... )
            
            >>> # Get with pagination
            >>> result = fgt.api.cmdb.user_radius.get(
            ...     start=0, count=100
            ... )
            
            >>> # Get schema information  
            >>> schema = fgt.api.cmdb.user_radius.get_schema()

        See Also:
            - post(): Create new user/radius object
            - put(): Update existing user/radius object
            - delete(): Remove user/radius object
            - exists(): Check if object exists
            - get_schema(): Get endpoint schema/metadata
        """
        params = payload_dict.copy() if payload_dict else {}
        
        # Add explicit query parameters
        if filter is not None:
            params["filter"] = filter
        if count is not None:
            params["count"] = count
        if start is not None:
            params["start"] = start
        
        if name:
            endpoint = "/user/radius/" + quote_path_param(name)
            unwrap_single = True
        else:
            endpoint = "/user/radius"
            unwrap_single = False
        
        return self._client.get(
            "cmdb", endpoint, params=params, vdom=vdom, unwrap_single=unwrap_single
        )

    def get_schema(
        self,
        vdom: str | None = None,
        format: str = "schema",
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Get schema/metadata for this endpoint.
        
        Returns the FortiOS schema definition including available fields,
        their types, required vs optional properties, enum values, nested
        structures, and default values.
        
        This queries the live firewall for its current schema, which may
        vary between FortiOS versions.
        
        Args:
            vdom: Virtual domain. None uses default VDOM.
            format: Schema format - "schema" (FortiOS native) or "json-schema" (JSON Schema standard).
                Defaults to "schema".
                
        Returns:
            Schema definition as dict. Returns Coroutine if using async client.
            
        Example:
            >>> # Get FortiOS native schema
            >>> schema = fgt.api.cmdb.user_radius.get_schema()
            >>> print(schema['results'])
            
            >>> # Get JSON Schema format (if supported)
            >>> json_schema = fgt.api.cmdb.user_radius.get_schema(format="json-schema")
        
        Note:
            Not all endpoints support all schema formats. The "schema" format
            is most widely supported.
        """
        return self.get(action=format, vdom=vdom)


    # ========================================================================
    # PUT Method
    # Type hints provided by CRUDEndpoint protocol (no local @overload needed)
    # ========================================================================
    
    def put(
        self,
        payload_dict: dict[str, Any] | None = None,
        name: str | None = None,
        server: str | None = None,
        secret: Any | None = None,
        secondary_server: str | None = None,
        secondary_secret: Any | None = None,
        tertiary_server: str | None = None,
        tertiary_secret: Any | None = None,
        timeout: int | None = None,
        status_ttl: int | None = None,
        all_usergroup: Literal["disable", "enable"] | None = None,
        use_management_vdom: Literal["enable", "disable"] | None = None,
        switch_controller_nas_ip_dynamic: Literal["enable", "disable"] | None = None,
        nas_ip: str | None = None,
        nas_id_type: Literal["legacy", "custom", "hostname"] | None = None,
        call_station_id_type: Literal["legacy", "IP", "MAC"] | None = None,
        nas_id: str | None = None,
        acct_interim_interval: int | None = None,
        radius_coa: Literal["enable", "disable"] | None = None,
        radius_port: int | None = None,
        h3c_compatibility: Literal["enable", "disable"] | None = None,
        auth_type: Literal["auto", "ms_chap_v2", "ms_chap", "chap", "pap"] | None = None,
        source_ip: str | None = None,
        source_ip_interface: str | None = None,
        username_case_sensitive: Literal["enable", "disable"] | None = None,
        group_override_attr_type: Literal["filter-Id", "class"] | None = None,
        class_: str | list[str] | list[dict[str, Any]] | None = None,
        password_renewal: Literal["enable", "disable"] | None = None,
        require_message_authenticator: Literal["enable", "disable"] | None = None,
        password_encoding: Literal["auto", "ISO-8859-1"] | None = None,
        mac_username_delimiter: Literal["hyphen", "single-hyphen", "colon", "none"] | None = None,
        mac_password_delimiter: Literal["hyphen", "single-hyphen", "colon", "none"] | None = None,
        mac_case: Literal["uppercase", "lowercase"] | None = None,
        acct_all_servers: Literal["enable", "disable"] | None = None,
        switch_controller_acct_fast_framedip_detect: int | None = None,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = None,
        interface: str | None = None,
        vrf_select: int | None = None,
        switch_controller_service_type: Literal["login", "framed", "callback-login", "callback-framed", "outbound", "administrative", "nas-prompt", "authenticate-only", "callback-nas-prompt", "call-check", "callback-administrative"] | list[str] | None = None,
        transport_protocol: Literal["udp", "tcp", "tls"] | None = None,
        tls_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = None,
        ca_cert: str | None = None,
        client_cert: str | None = None,
        server_identity_check: Literal["enable", "disable"] | None = None,
        account_key_processing: Literal["same", "strip"] | None = None,
        account_key_cert_field: Literal["othername", "rfc822name", "dnsname", "cn"] | None = None,
        rsso: Literal["enable", "disable"] | None = None,
        rsso_radius_server_port: int | None = None,
        rsso_radius_response: Literal["enable", "disable"] | None = None,
        rsso_validate_request_secret: Literal["enable", "disable"] | None = None,
        rsso_secret: Any | None = None,
        rsso_endpoint_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        rsso_endpoint_block_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        sso_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        sso_attribute_key: str | None = None,
        sso_attribute_value_override: Literal["enable", "disable"] | None = None,
        rsso_context_timeout: int | None = None,
        rsso_log_period: int | None = None,
        rsso_log_flags: Literal["protocol-error", "profile-missing", "accounting-stop-missed", "accounting-event", "endpoint-block", "radiusd-other", "none"] | list[str] | None = None,
        rsso_flush_ip_session: Literal["enable", "disable"] | None = None,
        rsso_ep_one_ip_only: Literal["enable", "disable"] | None = None,
        delimiter: Literal["plus", "comma"] | None = None,
        accounting_server: str | list[str] | list[dict[str, Any]] | None = None,
        q_action: Literal["move"] | None = None,
        q_before: str | None = None,
        q_after: str | None = None,
        q_scope: str | None = None,
        vdom: str | bool | None = None,
        error_mode: Literal["raise", "return", "print"] | None = None,
        error_format: Literal["detailed", "simple", "code_only"] | None = None,
    ):  # type: ignore[no-untyped-def]
        """
        Update existing user/radius object.

        Configure RADIUS server entries.

        Args:
            payload_dict: Object data as dict. Must include name (primary key).
            name: RADIUS server entry name.
            server: Primary RADIUS server CN domain name or IP address.
            secret: Pre-shared secret key used to access the primary RADIUS server.
            secondary_server: Secondary RADIUS CN domain name or IP address.
            secondary_secret: Secret key to access the secondary server.
            tertiary_server: Tertiary RADIUS CN domain name or IP address.
            tertiary_secret: Secret key to access the tertiary server.
            timeout: Time in seconds to retry connecting server.
            status_ttl: Time for which server reachability is cached so that when a server is unreachable, it will not be retried for at least this period of time (0 = cache disabled, default = 300).
            all_usergroup: Enable/disable automatically including this RADIUS server in all user groups.
            use_management_vdom: Enable/disable using management VDOM to send requests.
            switch_controller_nas_ip_dynamic: Enable/Disable switch-controller nas-ip dynamic to dynamically set nas-ip.
            nas_ip: IP address used to communicate with the RADIUS server and used as NAS-IP-Address and Called-Station-ID attributes.
            nas_id_type: NAS identifier type configuration (default = legacy).
            call_station_id_type: Calling & Called station identifier type configuration (default = legacy), this option is not available for 802.1x authentication. 
            nas_id: Custom NAS identifier.
            acct_interim_interval: Time in seconds between each accounting interim update message.
            radius_coa: Enable to allow a mechanism to change the attributes of an authentication, authorization, and accounting session after it is authenticated.
            radius_port: RADIUS service port number.
            h3c_compatibility: Enable/disable compatibility with the H3C, a mechanism that performs security checking for authentication.
            auth_type: Authentication methods/protocols permitted for this RADIUS server.
            source_ip: Source IP address for communications to the RADIUS server.
            source_ip_interface: Source interface for communication with the RADIUS server.
            username_case_sensitive: Enable/disable case sensitive user names.
            group_override_attr_type: RADIUS attribute type to override user group information.
            class_: Class attribute name(s).
            password_renewal: Enable/disable password renewal.
            require_message_authenticator: Require message authenticator in authentication response.
            password_encoding: Password encoding.
            mac_username_delimiter: MAC authentication username delimiter (default = hyphen).
            mac_password_delimiter: MAC authentication password delimiter (default = hyphen).
            mac_case: MAC authentication case (default = lowercase).
            acct_all_servers: Enable/disable sending of accounting messages to all configured servers (default = disable).
            switch_controller_acct_fast_framedip_detect: Switch controller accounting message Framed-IP detection from DHCP snooping (seconds, default=2).
            interface_select_method: Specify how to select outgoing interface to reach server.
            interface: Specify outgoing interface to reach server.
            vrf_select: VRF ID used for connection to server.
            switch_controller_service_type: RADIUS service type.
            transport_protocol: Transport protocol to be used (default = udp).
            tls_min_proto_version: Minimum supported protocol version for TLS connections (default is to follow system global setting).
            ca_cert: CA of server to trust under TLS.
            client_cert: Client certificate to use under TLS.
            server_identity_check: Enable/disable RADIUS server identity check (verify server domain name/IP address against the server certificate).
            account_key_processing: Account key processing operation. The FortiGate will keep either the whole domain or strip the domain from the subject identity.
            account_key_cert_field: Define subject identity field in certificate for user access right checking.
            rsso: Enable/disable RADIUS based single sign on feature.
            rsso_radius_server_port: UDP port to listen on for RADIUS Start and Stop records.
            rsso_radius_response: Enable/disable sending RADIUS response packets after receiving Start and Stop records.
            rsso_validate_request_secret: Enable/disable validating the RADIUS request shared secret in the Start or End record.
            rsso_secret: RADIUS secret used by the RADIUS accounting server.
            rsso_endpoint_attribute: RADIUS attributes used to extract the user end point identifier from the RADIUS Start record.
            rsso_endpoint_block_attribute: RADIUS attributes used to block a user.
            sso_attribute: RADIUS attribute that contains the profile group name to be extracted from the RADIUS Start record.
            sso_attribute_key: Key prefix for SSO group value in the SSO attribute.
            sso_attribute_value_override: Enable/disable override old attribute value with new value for the same endpoint.
            rsso_context_timeout: Time in seconds before the logged out user is removed from the "user context list" of logged on users.
            rsso_log_period: Time interval in seconds that group event log messages will be generated for dynamic profile events.
            rsso_log_flags: Events to log.
            rsso_flush_ip_session: Enable/disable flushing user IP sessions on RADIUS accounting Stop messages.
            rsso_ep_one_ip_only: Enable/disable the replacement of old IP addresses with new ones for the same endpoint on RADIUS accounting Start messages.
            delimiter: Configure delimiter to be used for separating profile group names in the SSO attribute (default = plus character "+").
            accounting_server: Additional accounting servers.
                Default format: [{'server': '192.168.1.10', 'secret': 'value', 'interface': 'value'}]
                Required format: List of dicts with keys: server, secret, interface
                  (String format not allowed due to multiple required fields)
            vdom: Virtual domain name.
            error_mode: Override client-level error_mode. "raise" raises exceptions, "return" returns error dict, "print" prints errors.
            error_format: Override client-level error_format. "detailed" provides full context, "simple" is concise, "code_only" returns just status code.

        Returns:
            FortiObject instance. Use .dict, .json, or .raw to access as dictionary.

        Raises:
            ValueError: If name is missing from payload

        Examples:
            >>> # Update specific fields
            >>> result = fgt.api.cmdb.user_radius.put(
            ...     name=1,
            ...     # ... fields to update
            ... )
            
            >>> # Update using payload dict
            >>> payload = {
            ...     "name": 1,
            ...     "field1": "new-value",
            ... }
            >>> result = fgt.api.cmdb.user_radius.put(payload_dict=payload)

        See Also:
            - post(): Create new object
            - set(): Intelligent create or update
        """
        # Apply normalization for table fields (supports flexible input formats)
        if class_ is not None:
            class_ = normalize_table_field(
                class_,
                mkey="name",
                required_fields=['name'],
                field_name="class_",
                example="[{'name': 'value'}]",
            )
        if accounting_server is not None:
            accounting_server = normalize_table_field(
                accounting_server,
                mkey="id",
                required_fields=['server', 'secret', 'interface'],
                field_name="accounting_server",
                example="[{'server': '192.168.1.10', 'secret': 'value', 'interface': 'value'}]",
            )
        
        # Apply normalization for multi-value option fields (space-separated strings)
        
        # Build payload using helper function
        # Note: auto_normalize=False because this endpoint has unitary fields
        # (like 'interface') that would be incorrectly converted to list format
        payload_data = build_api_payload(
            auto_normalize=False,
            name=name,
            server=server,
            secret=secret,
            secondary_server=secondary_server,
            secondary_secret=secondary_secret,
            tertiary_server=tertiary_server,
            tertiary_secret=tertiary_secret,
            timeout=timeout,
            status_ttl=status_ttl,
            all_usergroup=all_usergroup,
            use_management_vdom=use_management_vdom,
            switch_controller_nas_ip_dynamic=switch_controller_nas_ip_dynamic,
            nas_ip=nas_ip,
            nas_id_type=nas_id_type,
            call_station_id_type=call_station_id_type,
            nas_id=nas_id,
            acct_interim_interval=acct_interim_interval,
            radius_coa=radius_coa,
            radius_port=radius_port,
            h3c_compatibility=h3c_compatibility,
            auth_type=auth_type,
            source_ip=source_ip,
            source_ip_interface=source_ip_interface,
            username_case_sensitive=username_case_sensitive,
            group_override_attr_type=group_override_attr_type,
            class_=class_,
            password_renewal=password_renewal,
            require_message_authenticator=require_message_authenticator,
            password_encoding=password_encoding,
            mac_username_delimiter=mac_username_delimiter,
            mac_password_delimiter=mac_password_delimiter,
            mac_case=mac_case,
            acct_all_servers=acct_all_servers,
            switch_controller_acct_fast_framedip_detect=switch_controller_acct_fast_framedip_detect,
            interface_select_method=interface_select_method,
            interface=interface,
            vrf_select=vrf_select,
            switch_controller_service_type=switch_controller_service_type,
            transport_protocol=transport_protocol,
            tls_min_proto_version=tls_min_proto_version,
            ca_cert=ca_cert,
            client_cert=client_cert,
            server_identity_check=server_identity_check,
            account_key_processing=account_key_processing,
            account_key_cert_field=account_key_cert_field,
            rsso=rsso,
            rsso_radius_server_port=rsso_radius_server_port,
            rsso_radius_response=rsso_radius_response,
            rsso_validate_request_secret=rsso_validate_request_secret,
            rsso_secret=rsso_secret,
            rsso_endpoint_attribute=rsso_endpoint_attribute,
            rsso_endpoint_block_attribute=rsso_endpoint_block_attribute,
            sso_attribute=sso_attribute,
            sso_attribute_key=sso_attribute_key,
            sso_attribute_value_override=sso_attribute_value_override,
            rsso_context_timeout=rsso_context_timeout,
            rsso_log_period=rsso_log_period,
            rsso_log_flags=rsso_log_flags,
            rsso_flush_ip_session=rsso_flush_ip_session,
            rsso_ep_one_ip_only=rsso_ep_one_ip_only,
            delimiter=delimiter,
            accounting_server=accounting_server,
            data=payload_dict,
        )
        
        # Check for deprecated fields and warn users
        from ._helpers.radius import DEPRECATED_FIELDS
        if DEPRECATED_FIELDS:
            from hfortix_core import check_deprecated_fields
            check_deprecated_fields(
                payload=payload_data,
                deprecated_fields=DEPRECATED_FIELDS,
                endpoint="cmdb/user/radius",
            )
        
        name_value = payload_data.get("name")
        if not name_value:
            raise ValueError("name is required for PUT")
        endpoint = "/user/radius/" + quote_path_param(name_value)

        # Add explicit query parameters for PUT
        params: dict[str, Any] = {}
        if q_action is not None:
            params["action"] = q_action
        if q_before is not None:
            params["before"] = q_before
        if q_after is not None:
            params["after"] = q_after
        if q_scope is not None:
            params["scope"] = q_scope
        
        return self._client.put(
            "cmdb", endpoint, data=payload_data, params=params, vdom=vdom        )

    # ========================================================================
    # POST Method
    # Type hints provided by CRUDEndpoint protocol (no local @overload needed)
    # ========================================================================
    
    def post(
        self,
        payload_dict: dict[str, Any] | None = None,
        name: str | None = None,
        server: str | None = None,
        secret: Any | None = None,
        secondary_server: str | None = None,
        secondary_secret: Any | None = None,
        tertiary_server: str | None = None,
        tertiary_secret: Any | None = None,
        timeout: int | None = None,
        status_ttl: int | None = None,
        all_usergroup: Literal["disable", "enable"] | None = None,
        use_management_vdom: Literal["enable", "disable"] | None = None,
        switch_controller_nas_ip_dynamic: Literal["enable", "disable"] | None = None,
        nas_ip: str | None = None,
        nas_id_type: Literal["legacy", "custom", "hostname"] | None = None,
        call_station_id_type: Literal["legacy", "IP", "MAC"] | None = None,
        nas_id: str | None = None,
        acct_interim_interval: int | None = None,
        radius_coa: Literal["enable", "disable"] | None = None,
        radius_port: int | None = None,
        h3c_compatibility: Literal["enable", "disable"] | None = None,
        auth_type: Literal["auto", "ms_chap_v2", "ms_chap", "chap", "pap"] | None = None,
        source_ip: str | None = None,
        source_ip_interface: str | None = None,
        username_case_sensitive: Literal["enable", "disable"] | None = None,
        group_override_attr_type: Literal["filter-Id", "class"] | None = None,
        class_: str | list[str] | list[dict[str, Any]] | None = None,
        password_renewal: Literal["enable", "disable"] | None = None,
        require_message_authenticator: Literal["enable", "disable"] | None = None,
        password_encoding: Literal["auto", "ISO-8859-1"] | None = None,
        mac_username_delimiter: Literal["hyphen", "single-hyphen", "colon", "none"] | None = None,
        mac_password_delimiter: Literal["hyphen", "single-hyphen", "colon", "none"] | None = None,
        mac_case: Literal["uppercase", "lowercase"] | None = None,
        acct_all_servers: Literal["enable", "disable"] | None = None,
        switch_controller_acct_fast_framedip_detect: int | None = None,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = None,
        interface: str | None = None,
        vrf_select: int | None = None,
        switch_controller_service_type: Literal["login", "framed", "callback-login", "callback-framed", "outbound", "administrative", "nas-prompt", "authenticate-only", "callback-nas-prompt", "call-check", "callback-administrative"] | list[str] | None = None,
        transport_protocol: Literal["udp", "tcp", "tls"] | None = None,
        tls_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = None,
        ca_cert: str | None = None,
        client_cert: str | None = None,
        server_identity_check: Literal["enable", "disable"] | None = None,
        account_key_processing: Literal["same", "strip"] | None = None,
        account_key_cert_field: Literal["othername", "rfc822name", "dnsname", "cn"] | None = None,
        rsso: Literal["enable", "disable"] | None = None,
        rsso_radius_server_port: int | None = None,
        rsso_radius_response: Literal["enable", "disable"] | None = None,
        rsso_validate_request_secret: Literal["enable", "disable"] | None = None,
        rsso_secret: Any | None = None,
        rsso_endpoint_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        rsso_endpoint_block_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        sso_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        sso_attribute_key: str | None = None,
        sso_attribute_value_override: Literal["enable", "disable"] | None = None,
        rsso_context_timeout: int | None = None,
        rsso_log_period: int | None = None,
        rsso_log_flags: Literal["protocol-error", "profile-missing", "accounting-stop-missed", "accounting-event", "endpoint-block", "radiusd-other", "none"] | list[str] | None = None,
        rsso_flush_ip_session: Literal["enable", "disable"] | None = None,
        rsso_ep_one_ip_only: Literal["enable", "disable"] | None = None,
        delimiter: Literal["plus", "comma"] | None = None,
        accounting_server: str | list[str] | list[dict[str, Any]] | None = None,
        q_action: Literal["clone"] | None = None,
        q_nkey: str | None = None,
        q_scope: str | None = None,
        vdom: str | bool | None = None,
        error_mode: Literal["raise", "return", "print"] | None = None,
        error_format: Literal["detailed", "simple", "code_only"] | None = None,
    ):  # type: ignore[no-untyped-def]
        """
        Create new user/radius object.

        Configure RADIUS server entries.

        Args:
            payload_dict: Complete object data as dict. Alternative to individual parameters.
            name: RADIUS server entry name.
            server: Primary RADIUS server CN domain name or IP address.
            secret: Pre-shared secret key used to access the primary RADIUS server.
            secondary_server: Secondary RADIUS CN domain name or IP address.
            secondary_secret: Secret key to access the secondary server.
            tertiary_server: Tertiary RADIUS CN domain name or IP address.
            tertiary_secret: Secret key to access the tertiary server.
            timeout: Time in seconds to retry connecting server.
            status_ttl: Time for which server reachability is cached so that when a server is unreachable, it will not be retried for at least this period of time (0 = cache disabled, default = 300).
            all_usergroup: Enable/disable automatically including this RADIUS server in all user groups.
            use_management_vdom: Enable/disable using management VDOM to send requests.
            switch_controller_nas_ip_dynamic: Enable/Disable switch-controller nas-ip dynamic to dynamically set nas-ip.
            nas_ip: IP address used to communicate with the RADIUS server and used as NAS-IP-Address and Called-Station-ID attributes.
            nas_id_type: NAS identifier type configuration (default = legacy).
            call_station_id_type: Calling & Called station identifier type configuration (default = legacy), this option is not available for 802.1x authentication. 
            nas_id: Custom NAS identifier.
            acct_interim_interval: Time in seconds between each accounting interim update message.
            radius_coa: Enable to allow a mechanism to change the attributes of an authentication, authorization, and accounting session after it is authenticated.
            radius_port: RADIUS service port number.
            h3c_compatibility: Enable/disable compatibility with the H3C, a mechanism that performs security checking for authentication.
            auth_type: Authentication methods/protocols permitted for this RADIUS server.
            source_ip: Source IP address for communications to the RADIUS server.
            source_ip_interface: Source interface for communication with the RADIUS server.
            username_case_sensitive: Enable/disable case sensitive user names.
            group_override_attr_type: RADIUS attribute type to override user group information.
            class_: Class attribute name(s).
            password_renewal: Enable/disable password renewal.
            require_message_authenticator: Require message authenticator in authentication response.
            password_encoding: Password encoding.
            mac_username_delimiter: MAC authentication username delimiter (default = hyphen).
            mac_password_delimiter: MAC authentication password delimiter (default = hyphen).
            mac_case: MAC authentication case (default = lowercase).
            acct_all_servers: Enable/disable sending of accounting messages to all configured servers (default = disable).
            switch_controller_acct_fast_framedip_detect: Switch controller accounting message Framed-IP detection from DHCP snooping (seconds, default=2).
            interface_select_method: Specify how to select outgoing interface to reach server.
            interface: Specify outgoing interface to reach server.
            vrf_select: VRF ID used for connection to server.
            switch_controller_service_type: RADIUS service type.
            transport_protocol: Transport protocol to be used (default = udp).
            tls_min_proto_version: Minimum supported protocol version for TLS connections (default is to follow system global setting).
            ca_cert: CA of server to trust under TLS.
            client_cert: Client certificate to use under TLS.
            server_identity_check: Enable/disable RADIUS server identity check (verify server domain name/IP address against the server certificate).
            account_key_processing: Account key processing operation. The FortiGate will keep either the whole domain or strip the domain from the subject identity.
            account_key_cert_field: Define subject identity field in certificate for user access right checking.
            rsso: Enable/disable RADIUS based single sign on feature.
            rsso_radius_server_port: UDP port to listen on for RADIUS Start and Stop records.
            rsso_radius_response: Enable/disable sending RADIUS response packets after receiving Start and Stop records.
            rsso_validate_request_secret: Enable/disable validating the RADIUS request shared secret in the Start or End record.
            rsso_secret: RADIUS secret used by the RADIUS accounting server.
            rsso_endpoint_attribute: RADIUS attributes used to extract the user end point identifier from the RADIUS Start record.
            rsso_endpoint_block_attribute: RADIUS attributes used to block a user.
            sso_attribute: RADIUS attribute that contains the profile group name to be extracted from the RADIUS Start record.
            sso_attribute_key: Key prefix for SSO group value in the SSO attribute.
            sso_attribute_value_override: Enable/disable override old attribute value with new value for the same endpoint.
            rsso_context_timeout: Time in seconds before the logged out user is removed from the "user context list" of logged on users.
            rsso_log_period: Time interval in seconds that group event log messages will be generated for dynamic profile events.
            rsso_log_flags: Events to log.
            rsso_flush_ip_session: Enable/disable flushing user IP sessions on RADIUS accounting Stop messages.
            rsso_ep_one_ip_only: Enable/disable the replacement of old IP addresses with new ones for the same endpoint on RADIUS accounting Start messages.
            delimiter: Configure delimiter to be used for separating profile group names in the SSO attribute (default = plus character "+").
            accounting_server: Additional accounting servers.
                Default format: [{'server': '192.168.1.10', 'secret': 'value', 'interface': 'value'}]
                Required format: List of dicts with keys: server, secret, interface
                  (String format not allowed due to multiple required fields)
            vdom: Virtual domain name. Use True for global, string for specific VDOM.
            error_mode: Override client-level error_mode. "raise" raises exceptions, "return" returns error dict, "print" prints errors.
            error_format: Override client-level error_format. "detailed" provides full context, "simple" is concise, "code_only" returns just status code.

        Returns:
            FortiObject instance with created object. Use .dict, .json, or .raw to access as dictionary.

        Examples:
            >>> # Create using individual parameters
            >>> result = fgt.api.cmdb.user_radius.post(
            ...     name="example",
            ...     # ... other required fields
            ... )
            >>> print(f"Created name: {result['results']}")
            
            >>> # Create using payload dict
            >>> payload = Radius.defaults()  # Start with defaults
            >>> payload['name'] = 'my-object'
            >>> result = fgt.api.cmdb.user_radius.post(payload_dict=payload)

        Note:
            Required fields: {{ ", ".join(Radius.required_fields()) }}
            
            Use Radius.help('field_name') to get field details.

        See Also:
            - get(): Retrieve objects
            - put(): Update existing object
            - set(): Intelligent create or update
        """
        # Apply normalization for table fields (supports flexible input formats)
        if class_ is not None:
            class_ = normalize_table_field(
                class_,
                mkey="name",
                required_fields=['name'],
                field_name="class_",
                example="[{'name': 'value'}]",
            )
        if accounting_server is not None:
            accounting_server = normalize_table_field(
                accounting_server,
                mkey="id",
                required_fields=['server', 'secret', 'interface'],
                field_name="accounting_server",
                example="[{'server': '192.168.1.10', 'secret': 'value', 'interface': 'value'}]",
            )
        
        # Apply normalization for multi-value option fields (space-separated strings)
        
        # Build payload using helper function
        # Note: auto_normalize=False because this endpoint has unitary fields
        # (like 'interface') that would be incorrectly converted to list format
        payload_data = build_api_payload(
            auto_normalize=False,
            name=name,
            server=server,
            secret=secret,
            secondary_server=secondary_server,
            secondary_secret=secondary_secret,
            tertiary_server=tertiary_server,
            tertiary_secret=tertiary_secret,
            timeout=timeout,
            status_ttl=status_ttl,
            all_usergroup=all_usergroup,
            use_management_vdom=use_management_vdom,
            switch_controller_nas_ip_dynamic=switch_controller_nas_ip_dynamic,
            nas_ip=nas_ip,
            nas_id_type=nas_id_type,
            call_station_id_type=call_station_id_type,
            nas_id=nas_id,
            acct_interim_interval=acct_interim_interval,
            radius_coa=radius_coa,
            radius_port=radius_port,
            h3c_compatibility=h3c_compatibility,
            auth_type=auth_type,
            source_ip=source_ip,
            source_ip_interface=source_ip_interface,
            username_case_sensitive=username_case_sensitive,
            group_override_attr_type=group_override_attr_type,
            class_=class_,
            password_renewal=password_renewal,
            require_message_authenticator=require_message_authenticator,
            password_encoding=password_encoding,
            mac_username_delimiter=mac_username_delimiter,
            mac_password_delimiter=mac_password_delimiter,
            mac_case=mac_case,
            acct_all_servers=acct_all_servers,
            switch_controller_acct_fast_framedip_detect=switch_controller_acct_fast_framedip_detect,
            interface_select_method=interface_select_method,
            interface=interface,
            vrf_select=vrf_select,
            switch_controller_service_type=switch_controller_service_type,
            transport_protocol=transport_protocol,
            tls_min_proto_version=tls_min_proto_version,
            ca_cert=ca_cert,
            client_cert=client_cert,
            server_identity_check=server_identity_check,
            account_key_processing=account_key_processing,
            account_key_cert_field=account_key_cert_field,
            rsso=rsso,
            rsso_radius_server_port=rsso_radius_server_port,
            rsso_radius_response=rsso_radius_response,
            rsso_validate_request_secret=rsso_validate_request_secret,
            rsso_secret=rsso_secret,
            rsso_endpoint_attribute=rsso_endpoint_attribute,
            rsso_endpoint_block_attribute=rsso_endpoint_block_attribute,
            sso_attribute=sso_attribute,
            sso_attribute_key=sso_attribute_key,
            sso_attribute_value_override=sso_attribute_value_override,
            rsso_context_timeout=rsso_context_timeout,
            rsso_log_period=rsso_log_period,
            rsso_log_flags=rsso_log_flags,
            rsso_flush_ip_session=rsso_flush_ip_session,
            rsso_ep_one_ip_only=rsso_ep_one_ip_only,
            delimiter=delimiter,
            accounting_server=accounting_server,
            data=payload_dict,
        )

        # Check for deprecated fields and warn users
        from ._helpers.radius import DEPRECATED_FIELDS
        if DEPRECATED_FIELDS:
            from hfortix_core import check_deprecated_fields
            check_deprecated_fields(
                payload=payload_data,
                deprecated_fields=DEPRECATED_FIELDS,
                endpoint="cmdb/user/radius",
            )

        endpoint = "/user/radius"
        
        # Add explicit query parameters for POST
        params: dict[str, Any] = {}
        if q_action is not None:
            params["action"] = q_action
        if q_nkey is not None:
            params["nkey"] = q_nkey
        if q_scope is not None:
            params["scope"] = q_scope
        
        return self._client.post(
            "cmdb", endpoint, data=payload_data, params=params, vdom=vdom        )

    # ========================================================================
    # DELETE Method
    # Type hints provided by CRUDEndpoint protocol (no local @overload needed)
    # ========================================================================
    
    def delete(
        self,
        name: str | None = None,
        q_scope: str | None = None,
        vdom: str | bool | None = None,
        error_mode: Literal["raise", "return", "print"] | None = None,
        error_format: Literal["detailed", "simple", "code_only"] | None = None,
    ):  # type: ignore[no-untyped-def]
        """
        Delete user/radius object.

        Configure RADIUS server entries.

        Args:
            name: Primary key identifier
            vdom: Virtual domain name
            error_mode: Override client-level error_mode. "raise" raises exceptions, "return" returns error dict, "print" prints errors.
            error_format: Override client-level error_format. "detailed" provides full context, "simple" is concise, "code_only" returns just status code.

        Returns:
            FortiObject instance. Use .dict, .json, or .raw to access as dictionary

        Raises:
            ValueError: If name is not provided

        Examples:
            >>> # Delete specific object
            >>> result = fgt.api.cmdb.user_radius.delete(name=1)
            
            >>> # Check for errors
            >>> if result.get('status') != 'success':
            ...     print(f"Delete failed: {result.get('error')}")

        See Also:
            - exists(): Check if object exists before deleting
            - get(): Retrieve object to verify it exists
        """
        if not name:
            raise ValueError("name is required for DELETE")
        endpoint = "/user/radius/" + quote_path_param(name)

        # Add explicit query parameters for DELETE
        params: dict[str, Any] = {}
        if q_scope is not None:
            params["scope"] = q_scope
        
        return self._client.delete(
            "cmdb", endpoint, params=params, vdom=vdom        )

    def exists(
        self,
        name: str,
        vdom: str | bool | None = None,
    ) -> Union[bool, Coroutine[Any, Any, bool]]:
        """
        Check if user/radius object exists.

        Verifies whether an object exists by attempting to retrieve it and checking the response status.

        Args:
            name: Primary key identifier
            vdom: Virtual domain name

        Returns:
            True if object exists, False otherwise

        Examples:
            >>> # Check if object exists before operations
            >>> if fgt.api.cmdb.user_radius.exists(name=1):
            ...     print("Object exists")
            ... else:
            ...     print("Object not found")
            
            >>> # Conditional delete
            >>> if fgt.api.cmdb.user_radius.exists(name=1):
            ...     fgt.api.cmdb.user_radius.delete(name=1)

        See Also:
            - get(): Retrieve full object data
            - set(): Create or update automatically based on existence
        """
        # Use direct request with silent error handling to avoid logging 404s
        # This is expected behavior for exists() - 404 just means "doesn't exist"
        endpoint = "/user/radius"
        endpoint = f"{endpoint}/{quote_path_param(name)}"
        
        # Make request with silent=True to suppress 404 error logging
        # (404 is expected when checking existence - it just means "doesn't exist")
        # Use _wrapped_client to access the underlying HTTPClient directly
        # (self._client is ResponseProcessingClient, _wrapped_client is HTTPClient)
        try:
            result = self._client._wrapped_client.get(
                "cmdb",
                endpoint,
                params=None,
                vdom=vdom,
                raw_json=True,
                silent=True,
            )
            
            if isinstance(result, dict):
                # Synchronous response - check status
                return result.get("status") == "success"
            else:
                # Asynchronous response
                async def _check() -> bool:
                    r = await result
                    return r.get("status") == "success"
                return _check()
        except Exception:
            # Any error (404, network, etc.) means we can't confirm existence
            return False


    def set(
        self,
        payload_dict: dict[str, Any] | None = None,
        name: str | None = None,
        server: str | None = None,
        secret: Any | None = None,
        secondary_server: str | None = None,
        secondary_secret: Any | None = None,
        tertiary_server: str | None = None,
        tertiary_secret: Any | None = None,
        timeout: int | None = None,
        status_ttl: int | None = None,
        all_usergroup: Literal["disable", "enable"] | None = None,
        use_management_vdom: Literal["enable", "disable"] | None = None,
        switch_controller_nas_ip_dynamic: Literal["enable", "disable"] | None = None,
        nas_ip: str | None = None,
        nas_id_type: Literal["legacy", "custom", "hostname"] | None = None,
        call_station_id_type: Literal["legacy", "IP", "MAC"] | None = None,
        nas_id: str | None = None,
        acct_interim_interval: int | None = None,
        radius_coa: Literal["enable", "disable"] | None = None,
        radius_port: int | None = None,
        h3c_compatibility: Literal["enable", "disable"] | None = None,
        auth_type: Literal["auto", "ms_chap_v2", "ms_chap", "chap", "pap"] | None = None,
        source_ip: str | None = None,
        source_ip_interface: str | None = None,
        username_case_sensitive: Literal["enable", "disable"] | None = None,
        group_override_attr_type: Literal["filter-Id", "class"] | None = None,
        class_: str | list[str] | list[dict[str, Any]] | None = None,
        password_renewal: Literal["enable", "disable"] | None = None,
        require_message_authenticator: Literal["enable", "disable"] | None = None,
        password_encoding: Literal["auto", "ISO-8859-1"] | None = None,
        mac_username_delimiter: Literal["hyphen", "single-hyphen", "colon", "none"] | None = None,
        mac_password_delimiter: Literal["hyphen", "single-hyphen", "colon", "none"] | None = None,
        mac_case: Literal["uppercase", "lowercase"] | None = None,
        acct_all_servers: Literal["enable", "disable"] | None = None,
        switch_controller_acct_fast_framedip_detect: int | None = None,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = None,
        interface: str | None = None,
        vrf_select: int | None = None,
        switch_controller_service_type: Literal["login", "framed", "callback-login", "callback-framed", "outbound", "administrative", "nas-prompt", "authenticate-only", "callback-nas-prompt", "call-check", "callback-administrative"] | list[str] | list[dict[str, Any]] | None = None,
        transport_protocol: Literal["udp", "tcp", "tls"] | None = None,
        tls_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = None,
        ca_cert: str | None = None,
        client_cert: str | None = None,
        server_identity_check: Literal["enable", "disable"] | None = None,
        account_key_processing: Literal["same", "strip"] | None = None,
        account_key_cert_field: Literal["othername", "rfc822name", "dnsname", "cn"] | None = None,
        rsso: Literal["enable", "disable"] | None = None,
        rsso_radius_server_port: int | None = None,
        rsso_radius_response: Literal["enable", "disable"] | None = None,
        rsso_validate_request_secret: Literal["enable", "disable"] | None = None,
        rsso_secret: Any | None = None,
        rsso_endpoint_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        rsso_endpoint_block_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        sso_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"] | None = None,
        sso_attribute_key: str | None = None,
        sso_attribute_value_override: Literal["enable", "disable"] | None = None,
        rsso_context_timeout: int | None = None,
        rsso_log_period: int | None = None,
        rsso_log_flags: Literal["protocol-error", "profile-missing", "accounting-stop-missed", "accounting-event", "endpoint-block", "radiusd-other", "none"] | list[str] | list[dict[str, Any]] | None = None,
        rsso_flush_ip_session: Literal["enable", "disable"] | None = None,
        rsso_ep_one_ip_only: Literal["enable", "disable"] | None = None,
        delimiter: Literal["plus", "comma"] | None = None,
        accounting_server: str | list[str] | list[dict[str, Any]] | None = None,
        vdom: str | bool | None = None,
        error_mode: Literal["raise", "return", "print"] | None = None,
        error_format: Literal["detailed", "simple", "code_only"] | None = None,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Create or update user/radius object (intelligent operation).

        Automatically determines whether to create (POST) or update (PUT) based on
        whether the resource exists. Requires the primary key (name) in the payload.

        Args:
            payload_dict: Resource data including name (primary key)
            name: Field name
            server: Field server
            secret: Field secret
            secondary_server: Field secondary-server
            secondary_secret: Field secondary-secret
            tertiary_server: Field tertiary-server
            tertiary_secret: Field tertiary-secret
            timeout: Field timeout
            status_ttl: Field status-ttl
            all_usergroup: Field all-usergroup
            use_management_vdom: Field use-management-vdom
            switch_controller_nas_ip_dynamic: Field switch-controller-nas-ip-dynamic
            nas_ip: Field nas-ip
            nas_id_type: Field nas-id-type
            call_station_id_type: Field call-station-id-type
            nas_id: Field nas-id
            acct_interim_interval: Field acct-interim-interval
            radius_coa: Field radius-coa
            radius_port: Field radius-port
            h3c_compatibility: Field h3c-compatibility
            auth_type: Field auth-type
            source_ip: Field source-ip
            source_ip_interface: Field source-ip-interface
            username_case_sensitive: Field username-case-sensitive
            group_override_attr_type: Field group-override-attr-type
            class_: Field class
            password_renewal: Field password-renewal
            require_message_authenticator: Field require-message-authenticator
            password_encoding: Field password-encoding
            mac_username_delimiter: Field mac-username-delimiter
            mac_password_delimiter: Field mac-password-delimiter
            mac_case: Field mac-case
            acct_all_servers: Field acct-all-servers
            switch_controller_acct_fast_framedip_detect: Field switch-controller-acct-fast-framedip-detect
            interface_select_method: Field interface-select-method
            interface: Field interface
            vrf_select: Field vrf-select
            switch_controller_service_type: Field switch-controller-service-type
            transport_protocol: Field transport-protocol
            tls_min_proto_version: Field tls-min-proto-version
            ca_cert: Field ca-cert
            client_cert: Field client-cert
            server_identity_check: Field server-identity-check
            account_key_processing: Field account-key-processing
            account_key_cert_field: Field account-key-cert-field
            rsso: Field rsso
            rsso_radius_server_port: Field rsso-radius-server-port
            rsso_radius_response: Field rsso-radius-response
            rsso_validate_request_secret: Field rsso-validate-request-secret
            rsso_secret: Field rsso-secret
            rsso_endpoint_attribute: Field rsso-endpoint-attribute
            rsso_endpoint_block_attribute: Field rsso-endpoint-block-attribute
            sso_attribute: Field sso-attribute
            sso_attribute_key: Field sso-attribute-key
            sso_attribute_value_override: Field sso-attribute-value-override
            rsso_context_timeout: Field rsso-context-timeout
            rsso_log_period: Field rsso-log-period
            rsso_log_flags: Field rsso-log-flags
            rsso_flush_ip_session: Field rsso-flush-ip-session
            rsso_ep_one_ip_only: Field rsso-ep-one-ip-only
            delimiter: Field delimiter
            accounting_server: Field accounting-server
            vdom: Virtual domain name
            **kwargs: Additional parameters passed to PUT or POST

        Returns:
            FortiObject instance. Use .dict, .json, or .raw to access as dictionary

        Raises:
            ValueError: If name is missing from payload

        Examples:
            >>> # Intelligent create or update using field parameters
            >>> result = fgt.api.cmdb.user_radius.set(
            ...     name=1,
            ...     # ... other fields
            ... )
            
            >>> # Or using payload dict
            >>> payload = {
            ...     "name": 1,
            ...     "field1": "value1",
            ...     "field2": "value2",
            ... }
            >>> result = fgt.api.cmdb.user_radius.set(payload_dict=payload)
            >>> # Will POST if object doesn't exist, PUT if it does
            
            >>> # Idempotent configuration
            >>> for obj_data in configuration_list:
            ...     fgt.api.cmdb.user_radius.set(payload_dict=obj_data)
            >>> # Safely applies configuration regardless of current state

        Note:
            This method internally calls exists() then either post() or put().
            For performance-critical code with known state, call post() or put() directly.

        See Also:
            - post(): Create new object
            - put(): Update existing object
            - exists(): Check existence manually
        """
        # Apply normalization for table fields (supports flexible input formats)
        if class_ is not None:
            class_ = normalize_table_field(
                class_,
                mkey="name",
                required_fields=['name'],
                field_name="class_",
                example="[{'name': 'value'}]",
            )
        if accounting_server is not None:
            accounting_server = normalize_table_field(
                accounting_server,
                mkey="id",
                required_fields=['server', 'secret', 'interface'],
                field_name="accounting_server",
                example="[{'server': '192.168.1.10', 'secret': 'value', 'interface': 'value'}]",
            )
        
        # Apply normalization for multi-value option fields (space-separated strings)
        
        # Build payload using helper function
        # Note: auto_normalize=False because this endpoint has unitary fields
        # (like 'interface') that would be incorrectly converted to list format
        payload_data = build_api_payload(
            auto_normalize=False,
            name=name,
            server=server,
            secret=secret,
            secondary_server=secondary_server,
            secondary_secret=secondary_secret,
            tertiary_server=tertiary_server,
            tertiary_secret=tertiary_secret,
            timeout=timeout,
            status_ttl=status_ttl,
            all_usergroup=all_usergroup,
            use_management_vdom=use_management_vdom,
            switch_controller_nas_ip_dynamic=switch_controller_nas_ip_dynamic,
            nas_ip=nas_ip,
            nas_id_type=nas_id_type,
            call_station_id_type=call_station_id_type,
            nas_id=nas_id,
            acct_interim_interval=acct_interim_interval,
            radius_coa=radius_coa,
            radius_port=radius_port,
            h3c_compatibility=h3c_compatibility,
            auth_type=auth_type,
            source_ip=source_ip,
            source_ip_interface=source_ip_interface,
            username_case_sensitive=username_case_sensitive,
            group_override_attr_type=group_override_attr_type,
            class_=class_,
            password_renewal=password_renewal,
            require_message_authenticator=require_message_authenticator,
            password_encoding=password_encoding,
            mac_username_delimiter=mac_username_delimiter,
            mac_password_delimiter=mac_password_delimiter,
            mac_case=mac_case,
            acct_all_servers=acct_all_servers,
            switch_controller_acct_fast_framedip_detect=switch_controller_acct_fast_framedip_detect,
            interface_select_method=interface_select_method,
            interface=interface,
            vrf_select=vrf_select,
            switch_controller_service_type=switch_controller_service_type,
            transport_protocol=transport_protocol,
            tls_min_proto_version=tls_min_proto_version,
            ca_cert=ca_cert,
            client_cert=client_cert,
            server_identity_check=server_identity_check,
            account_key_processing=account_key_processing,
            account_key_cert_field=account_key_cert_field,
            rsso=rsso,
            rsso_radius_server_port=rsso_radius_server_port,
            rsso_radius_response=rsso_radius_response,
            rsso_validate_request_secret=rsso_validate_request_secret,
            rsso_secret=rsso_secret,
            rsso_endpoint_attribute=rsso_endpoint_attribute,
            rsso_endpoint_block_attribute=rsso_endpoint_block_attribute,
            sso_attribute=sso_attribute,
            sso_attribute_key=sso_attribute_key,
            sso_attribute_value_override=sso_attribute_value_override,
            rsso_context_timeout=rsso_context_timeout,
            rsso_log_period=rsso_log_period,
            rsso_log_flags=rsso_log_flags,
            rsso_flush_ip_session=rsso_flush_ip_session,
            rsso_ep_one_ip_only=rsso_ep_one_ip_only,
            delimiter=delimiter,
            accounting_server=accounting_server,
            data=payload_dict,
        )
        
        mkey_value = payload_data.get("name")
        if not mkey_value:
            raise ValueError("name is required for set()")
        
        # Check if resource exists
        if self.exists(name=mkey_value, vdom=vdom):
            # Update existing resource
            return self.put(payload_dict=payload_data, vdom=vdom, **kwargs)
        else:
            # Create new resource
            return self.post(payload_dict=payload_data, vdom=vdom, **kwargs)

    # ========================================================================
    # Action: Move
    # ========================================================================
    
    def move(
        self,
        name: str,
        action: Literal["before", "after"],
        reference_name: str,
        vdom: str | bool | None = None,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Move user/radius object to a new position.
        
        Reorders objects by moving one before or after another.
        
        Args:
            name: Identifier of object to move
            action: Move "before" or "after" reference object
            reference_name: Identifier of reference object
            vdom: Virtual domain name
            **kwargs: Additional parameters
            
        Returns:
            API response dictionary
            
        Example:
            >>> # Move policy 100 before policy 50
            >>> fgt.api.cmdb.user_radius.move(
            ...     name=100,
            ...     action="before",
            ...     reference_name=50
            ... )
        """
        return self._client.request(
            method="PUT",
            path=f"/api/v2/cmdb/user/radius",
            params={
                "name": name,
                "action": "move",
                action: reference_name,
                "vdom": vdom,
                **kwargs,
            },
        )

    # ========================================================================
    # Action: Clone
    # ========================================================================
    
    def clone(
        self,
        name: str,
        new_name: str,
        vdom: str | bool | None = None,
        **kwargs: Any,
    ) -> Union[dict[str, Any], Coroutine[Any, Any, dict[str, Any]]]:
        """
        Clone user/radius object.
        
        Creates a copy of an existing object with a new identifier.
        
        Args:
            name: Identifier of object to clone
            new_name: Identifier for the cloned object
            vdom: Virtual domain name
            **kwargs: Additional parameters
            
        Returns:
            API response dictionary
            
        Example:
            >>> # Clone an existing object
            >>> fgt.api.cmdb.user_radius.clone(
            ...     name=1,
            ...     new_name=100
            ... )
        """
        return self._client.request(
            method="POST",
            path=f"/api/v2/cmdb/user/radius",
            params={
                "name": name,
                "new_name": new_name,
                "action": "clone",
                "vdom": vdom,
                **kwargs,
            },
        )





"""Type stubs for VMLICENSE category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .download import Download
    from .download_eval import DownloadEval
    from .upload import Upload

__all__ = [
    "Download",
    "DownloadEval",
    "Upload",
    "Vmlicense",
]


class Vmlicense:
    """VMLICENSE API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    download: Download
    download_eval: DownloadEval
    upload: Upload

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize vmlicense category with HTTP client."""
        ...



"""Validation helpers for switch_controller/location - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Unique location item name.
    "address-civic": "string",  # Configure location civic address.
    "coordinates": "string",  # Configure location GPS coordinates.
    "elin-number": "string",  # Configure location ELIN number.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Unique location item name.",
    "address-civic": "Configure location civic address.",
    "coordinates": "Configure location GPS coordinates.",
    "elin-number": "Configure location ELIN number.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 63},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "address-civic": {
        "additional": {
            "type": "string",
            "help": "Location additional details.",
            "default": "",
            "max_length": 47,
        },
        "additional-code": {
            "type": "string",
            "help": "Location additional code details.",
            "default": "",
            "max_length": 47,
        },
        "block": {
            "type": "string",
            "help": "Location block details.",
            "default": "",
            "max_length": 47,
        },
        "branch-road": {
            "type": "string",
            "help": "Location branch road details.",
            "default": "",
            "max_length": 47,
        },
        "building": {
            "type": "string",
            "help": "Location building details.",
            "default": "",
            "max_length": 47,
        },
        "city": {
            "type": "string",
            "help": "Location city details.",
            "default": "",
            "max_length": 47,
        },
        "city-division": {
            "type": "string",
            "help": "Location city division details.",
            "default": "",
            "max_length": 47,
        },
        "country": {
            "type": "string",
            "help": "The two-letter ISO 3166 country code in capital ASCII letters eg. US, CA, DK, DE.",
            "required": True,
            "default": "",
            "max_length": 47,
        },
        "country-subdivision": {
            "type": "string",
            "help": "National subdivisions (state, canton, region, province, or prefecture).",
            "default": "",
            "max_length": 47,
        },
        "county": {
            "type": "string",
            "help": "County, parish, gun (JP), or district (IN).",
            "default": "",
            "max_length": 47,
        },
        "direction": {
            "type": "string",
            "help": "Leading street direction.",
            "default": "",
            "max_length": 47,
        },
        "floor": {
            "type": "string",
            "help": "Floor.",
            "default": "",
            "max_length": 47,
        },
        "landmark": {
            "type": "string",
            "help": "Landmark or vanity address.",
            "default": "",
            "max_length": 47,
        },
        "language": {
            "type": "string",
            "help": "Language.",
            "default": "",
            "max_length": 47,
        },
        "name": {
            "type": "string",
            "help": "Name (residence and office occupant).",
            "default": "",
            "max_length": 47,
        },
        "number": {
            "type": "string",
            "help": "House number.",
            "default": "",
            "max_length": 47,
        },
        "number-suffix": {
            "type": "string",
            "help": "House number suffix.",
            "default": "",
            "max_length": 47,
        },
        "place-type": {
            "type": "string",
            "help": "Place type.",
            "default": "",
            "max_length": 47,
        },
        "post-office-box": {
            "type": "string",
            "help": "Post office box.",
            "default": "",
            "max_length": 47,
        },
        "postal-community": {
            "type": "string",
            "help": "Postal community name.",
            "default": "",
            "max_length": 47,
        },
        "primary-road": {
            "type": "string",
            "help": "Primary road name.",
            "default": "",
            "max_length": 47,
        },
        "road-section": {
            "type": "string",
            "help": "Road section.",
            "default": "",
            "max_length": 47,
        },
        "room": {
            "type": "string",
            "help": "Room number.",
            "default": "",
            "max_length": 47,
        },
        "script": {
            "type": "string",
            "help": "Script used to present the address information.",
            "default": "",
            "max_length": 47,
        },
        "seat": {
            "type": "string",
            "help": "Seat number.",
            "default": "",
            "max_length": 47,
        },
        "street": {
            "type": "string",
            "help": "Street.",
            "default": "",
            "max_length": 47,
        },
        "street-name-post-mod": {
            "type": "string",
            "help": "Street name post modifier.",
            "default": "",
            "max_length": 47,
        },
        "street-name-pre-mod": {
            "type": "string",
            "help": "Street name pre modifier.",
            "default": "",
            "max_length": 47,
        },
        "street-suffix": {
            "type": "string",
            "help": "Street suffix.",
            "default": "",
            "max_length": 47,
        },
        "sub-branch-road": {
            "type": "string",
            "help": "Sub branch road name.",
            "default": "",
            "max_length": 47,
        },
        "trailing-str-suffix": {
            "type": "string",
            "help": "Trailing street suffix.",
            "default": "",
            "max_length": 47,
        },
        "unit": {
            "type": "string",
            "help": "Unit (apartment, suite).",
            "default": "",
            "max_length": 47,
        },
        "zip": {
            "type": "string",
            "help": "Postal/zip code.",
            "default": "",
            "max_length": 47,
        },
        "parent-key": {
            "type": "string",
            "help": "Parent key name.",
            "default": "",
            "max_length": 63,
        },
    },
    "coordinates": {
        "altitude": {
            "type": "string",
            "help": "Plus or minus floating point number. For example, 117.47.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "altitude-unit": {
            "type": "option",
            "help": "Configure the unit for which the altitude is to (m = meters, f = floors of a building).",
            "required": True,
            "default": "m",
            "options": ["m", "f"],
        },
        "datum": {
            "type": "option",
            "help": "WGS84, NAD83, NAD83/MLLW.",
            "required": True,
            "default": "WGS84",
            "options": ["WGS84", "NAD83", "NAD83/MLLW"],
        },
        "latitude": {
            "type": "string",
            "help": "Floating point starting with +/- or ending with (N or S). For example, +/-16.67 or 16.67N.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "longitude": {
            "type": "string",
            "help": "Floating point starting with +/- or ending with (N or S). For example, +/-26.789 or 26.789E.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "parent-key": {
            "type": "string",
            "help": "Parent key name.",
            "default": "",
            "max_length": 63,
        },
    },
    "elin-number": {
        "elin-num": {
            "type": "string",
            "help": "Configure ELIN callback number.",
            "default": "",
            "max_length": 31,
        },
        "parent-key": {
            "type": "string",
            "help": "Parent key name.",
            "default": "",
            "max_length": 63,
        },
    },
}


# Valid enum values from API documentation
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_switch_controller_location_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for switch_controller/location."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_switch_controller_location_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new switch_controller/location object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_switch_controller_location_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update switch_controller/location."""
    # Validate enum values using central function

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "switch_controller/location",
    "category": "cmdb",
    "api_path": "switch-controller/location",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure FortiSwitch location services.",
    "total_fields": 4,
    "required_fields_count": 0,
    "fields_with_defaults_count": 1,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""Validation helpers for vpn/kmip_server - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "server-list",  # KMIP server list.
    "username",  # User name to use for connectivity to the KMIP server.
    "password",  # Password to use for connectivity to the KMIP server.
    "interface",  # Specify outgoing interface to reach server.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "username": "",
    "ssl-min-proto-version": "default",
    "server-identity-check": "disable",
    "interface-select-method": "auto",
    "interface": "",
    "vrf-select": 0,
    "source-ip": "",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # KMIP server entry name.
    "server-list": "string",  # KMIP server list.
    "username": "string",  # User name to use for connectivity to the KMIP server.
    "password": "password",  # Password to use for connectivity to the KMIP server.
    "ssl-min-proto-version": "option",  # Minimum supported protocol version for SSL/TLS connections (
    "server-identity-check": "option",  # Enable/disable KMIP server identity check (verify server FQD
    "interface-select-method": "option",  # Specify how to select outgoing interface to reach server.
    "interface": "string",  # Specify outgoing interface to reach server.
    "vrf-select": "integer",  # VRF ID used for connection to server.
    "source-ip": "string",  # FortiGate IP address to be used for communication with the K
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "KMIP server entry name.",
    "server-list": "KMIP server list.",
    "username": "User name to use for connectivity to the KMIP server.",
    "password": "Password to use for connectivity to the KMIP server.",
    "ssl-min-proto-version": "Minimum supported protocol version for SSL/TLS connections (default is to follow system global setting).",
    "server-identity-check": "Enable/disable KMIP server identity check (verify server FQDN/IP address against the server certificate).",
    "interface-select-method": "Specify how to select outgoing interface to reach server.",
    "interface": "Specify outgoing interface to reach server.",
    "vrf-select": "VRF ID used for connection to server.",
    "source-ip": "FortiGate IP address to be used for communication with the KMIP server.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "username": {"type": "string", "max_length": 63},
    "interface": {"type": "string", "max_length": 15},
    "vrf-select": {"type": "integer", "min": 0, "max": 511},
    "source-ip": {"type": "string", "max_length": 63},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "server-list": {
        "id": {
            "type": "integer",
            "help": "ID",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable KMIP server.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "server": {
            "type": "string",
            "help": "KMIP server FQDN or IP address.",
            "required": True,
            "default": "",
            "max_length": 63,
        },
        "port": {
            "type": "integer",
            "help": "KMIP server port.",
            "required": True,
            "default": 5696,
            "min_value": 0,
            "max_value": 65535,
        },
        "cert": {
            "type": "string",
            "help": "Client certificate to use for connectivity to the KMIP server.",
            "default": "",
            "max_length": 35,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_SSL_MIN_PROTO_VERSION = [
    "default",
    "SSLv3",
    "TLSv1",
    "TLSv1-1",
    "TLSv1-2",
    "TLSv1-3",
]
VALID_BODY_SERVER_IDENTITY_CHECK = [
    "enable",
    "disable",
]
VALID_BODY_INTERFACE_SELECT_METHOD = [
    "auto",
    "sdwan",
    "specify",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_vpn_kmip_server_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for vpn/kmip_server."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_vpn_kmip_server_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new vpn/kmip_server object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-proto-version",
            payload["ssl-min-proto-version"],
            VALID_BODY_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-identity-check" in payload:
        is_valid, error = _validate_enum_field(
            "server-identity-check",
            payload["server-identity-check"],
            VALID_BODY_SERVER_IDENTITY_CHECK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "interface-select-method" in payload:
        is_valid, error = _validate_enum_field(
            "interface-select-method",
            payload["interface-select-method"],
            VALID_BODY_INTERFACE_SELECT_METHOD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_vpn_kmip_server_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update vpn/kmip_server."""
    # Validate enum values using central function
    if "ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-proto-version",
            payload["ssl-min-proto-version"],
            VALID_BODY_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-identity-check" in payload:
        is_valid, error = _validate_enum_field(
            "server-identity-check",
            payload["server-identity-check"],
            VALID_BODY_SERVER_IDENTITY_CHECK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "interface-select-method" in payload:
        is_valid, error = _validate_enum_field(
            "interface-select-method",
            payload["interface-select-method"],
            VALID_BODY_INTERFACE_SELECT_METHOD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "vpn/kmip_server",
    "category": "cmdb",
    "api_path": "vpn/kmip-server",
    "mkey": "name",
    "mkey_type": "string",
    "help": "KMIP server entry configuration.",
    "total_fields": 10,
    "required_fields_count": 4,
    "fields_with_defaults_count": 8,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - system/speed_test_setting

Runtime validation models for system/speed_test_setting configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SpeedTestSettingModel(BaseModel):
    """
    Pydantic model for system/speed_test_setting configuration.
    
    Configure speed test setting.
    
    Validation Rules:        - latency_threshold: min=0 max=2000 pattern=        - multiple_tcp_stream: min=1 max=64 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    latency_threshold: int | None = Field(ge=0, le=2000, default=60, description="Speed test latency threshold in milliseconds (0 - 2000, default = 60) for the Auto mode. If the latency exceeds this threshold, the speed test will use the UDP protocol; otherwise, it will use the TCP protocol.")    
    multiple_tcp_stream: int | None = Field(ge=1, le=64, default=4, description="Number of parallel client streams (1 - 64, default = 4) for the TCP protocol to run during the speed test.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SpeedTestSettingModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SpeedTestSettingModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.793460Z
# ============================================================================


"""FortiOS CMDB - Wifi category"""

from . import ap_profile
from . import client as client_ns
from . import euclid
from . import firmware
from . import managed_ap
from . import nac_device
from . import network
from . import region_image
from . import rogue_ap
from . import spectrum
from . import ssid
from . import vlan_probe
from .ap_channels import ApChannels
from .ap_names import ApNames
from .ap_status import ApStatus
from .interfering_ap import InterferingAp
from .matched_devices import MatchedDevices
from .meta import Meta
from .station_capability import StationCapability
from .statistics import Statistics
from .unassociated_devices import UnassociatedDevices

__all__ = [
    "ApChannels",
    "ApNames",
    "ApProfile",
    "ApStatus",
    "Client",
    "Euclid",
    "Firmware",
    "InterferingAp",
    "ManagedAp",
    "MatchedDevices",
    "Meta",
    "NacDevice",
    "Network",
    "RegionImage",
    "RogueAp",
    "Spectrum",
    "Ssid",
    "StationCapability",
    "Statistics",
    "UnassociatedDevices",
    "VlanProbe",
    "Wifi",
]


class Wifi:
    """Wifi endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Wifi endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.ap_profile = ap_profile.ApProfile(client)
        self.client = client_ns.Client(client)
        self.euclid = euclid.Euclid(client)
        self.firmware = firmware.Firmware(client)
        self.managed_ap = managed_ap.ManagedAp(client)
        self.nac_device = nac_device.NacDevice(client)
        self.network = network.Network(client)
        self.region_image = region_image.RegionImage(client)
        self.rogue_ap = rogue_ap.RogueAp(client)
        self.spectrum = spectrum.Spectrum(client)
        self.ssid = ssid.Ssid(client)
        self.vlan_probe = vlan_probe.VlanProbe(client)
        self.ap_channels = ApChannels(client)
        self.ap_names = ApNames(client)
        self.ap_status = ApStatus(client)
        self.interfering_ap = InterferingAp(client)
        self.matched_devices = MatchedDevices(client)
        self.meta = Meta(client)
        self.station_capability = StationCapability(client)
        self.statistics = Statistics(client)
        self.unassociated_devices = UnassociatedDevices(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: switch_controller/managed_switch
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ManagedSwitchRouteoffloadrouterItem:
    """Nested item for route-offload-router field - supports attribute access."""
    vlan_name: str
    router_ip: str


class ManagedSwitchVlanItem:
    """Nested item for vlan field - supports attribute access."""
    vlan_name: str
    assignment_priority: int


class ManagedSwitchPortsItem:
    """Nested item for ports field - supports attribute access."""
    port_name: str
    port_owner: str
    switch_id: str
    speed: Literal["10half", "10full", "100half", "100full", "1000full", "10000full", "auto", "1000auto", "1000full-fiber", "40000full", "auto-module", "100FX-half", "100FX-full", "100000full", "2500auto", "2500full", "25000full", "50000full", "10000cr", "10000sr", "100000sr4", "100000cr4", "40000sr4", "40000cr4", "40000auto", "25000cr", "25000sr", "50000cr", "50000sr", "5000auto", "sgmii-auto"]
    status: Literal["up", "down"]
    poe_status: Literal["enable", "disable"]
    ip_source_guard: Literal["disable", "enable"]
    ptp_status: Literal["disable", "enable"]
    ptp_policy: str
    aggregator_mode: Literal["bandwidth", "count"]
    flapguard: Literal["enable", "disable"]
    flap_rate: int
    flap_duration: int
    flap_timeout: int
    rpvst_port: Literal["disabled", "enabled"]
    poe_pre_standard_detection: Literal["enable", "disable"]
    port_number: int
    port_prefix_type: int
    fortilink_port: int
    poe_capable: int
    pd_capable: int
    stacking_port: int
    p2p_port: int
    mclag_icl_port: int
    authenticated_port: int
    restricted_auth_port: int
    encrypted_port: int
    fiber_port: int
    media_type: str
    poe_standard: str
    poe_max_power: str
    poe_mode_bt_cabable: int
    poe_port_mode: Literal["ieee802-3af", "ieee802-3at", "ieee802-3bt"]
    poe_port_priority: Literal["critical-priority", "high-priority", "low-priority", "medium-priority"]
    poe_port_power: Literal["normal", "perpetual", "perpetual-fast"]
    flags: int
    isl_local_trunk_name: str
    isl_peer_port_name: str
    isl_peer_device_name: str
    isl_peer_device_sn: str
    fgt_peer_port_name: str
    fgt_peer_device_name: str
    vlan: str
    allowed_vlans_all: Literal["enable", "disable"]
    allowed_vlans: str
    untagged_vlans: str
    type: Literal["physical", "trunk"]
    access_mode: Literal["dynamic", "nac", "static"]
    matched_dpp_policy: str
    matched_dpp_intf_tags: str
    acl_group: str
    fortiswitch_acls: str
    dhcp_snooping: Literal["untrusted", "trusted"]
    dhcp_snoop_option82_trust: Literal["enable", "disable"]
    dhcp_snoop_option82_override: str
    arp_inspection_trust: Literal["untrusted", "trusted"]
    igmp_snooping_flood_reports: Literal["enable", "disable"]
    mcast_snooping_flood_traffic: Literal["enable", "disable"]
    stp_state: Literal["enabled", "disabled"]
    stp_root_guard: Literal["enabled", "disabled"]
    stp_bpdu_guard: Literal["enabled", "disabled"]
    stp_bpdu_guard_timeout: int
    edge_port: Literal["enable", "disable"]
    discard_mode: Literal["none", "all-untagged", "all-tagged"]
    packet_sampler: Literal["enabled", "disabled"]
    packet_sample_rate: int
    sflow_counter_interval: int
    sample_direction: Literal["tx", "rx", "both"]
    fec_capable: int
    fec_state: Literal["disabled", "cl74", "cl91", "detect-by-module"]
    flow_control: Literal["disable", "tx", "rx", "both"]
    pause_meter: int
    pause_meter_resume: Literal["75%", "50%", "25%"]
    loop_guard: Literal["enabled", "disabled"]
    loop_guard_timeout: int
    port_policy: str
    qos_policy: str
    storm_control_policy: str
    port_security_policy: str
    export_to_pool: str
    interface_tags: str
    learning_limit: int
    sticky_mac: Literal["enable", "disable"]
    lldp_status: Literal["disable", "rx-only", "tx-only", "tx-rx"]
    lldp_profile: str
    export_to: str
    mac_addr: str
    allow_arp_monitor: Literal["disable", "enable"]
    qnq: str
    log_mac_event: Literal["disable", "enable"]
    port_selection_criteria: Literal["src-mac", "dst-mac", "src-dst-mac", "src-ip", "dst-ip", "src-dst-ip"]
    description: str
    lacp_speed: Literal["slow", "fast"]
    mode: Literal["static", "lacp-passive", "lacp-active"]
    bundle: Literal["enable", "disable"]
    member_withdrawal_behavior: Literal["forward", "block"]
    mclag: Literal["enable", "disable"]
    min_bundle: int
    max_bundle: int
    members: str
    fallback_port: str


class ManagedSwitchIpsourceguardItem:
    """Nested item for ip-source-guard field - supports attribute access."""
    port: str
    description: str
    binding_entry: str


class ManagedSwitchStpinstanceItem:
    """Nested item for stp-instance field - supports attribute access."""
    id: str
    priority: Literal["0", "4096", "8192", "12288", "16384", "20480", "24576", "28672", "32768", "36864", "40960", "45056", "49152", "53248", "57344", "61440"]


class ManagedSwitchSnmpcommunityItem:
    """Nested item for snmp-community field - supports attribute access."""
    id: int
    name: str
    status: Literal["disable", "enable"]
    hosts: str
    query_v1_status: Literal["disable", "enable"]
    query_v1_port: int
    query_v2c_status: Literal["disable", "enable"]
    query_v2c_port: int
    trap_v1_status: Literal["disable", "enable"]
    trap_v1_lport: int
    trap_v1_rport: int
    trap_v2c_status: Literal["disable", "enable"]
    trap_v2c_lport: int
    trap_v2c_rport: int
    events: Literal["cpu-high", "mem-low", "log-full", "intf-ip", "ent-conf-change", "l2mac"]


class ManagedSwitchSnmpuserItem:
    """Nested item for snmp-user field - supports attribute access."""
    name: str
    queries: Literal["disable", "enable"]
    query_port: int
    security_level: Literal["no-auth-no-priv", "auth-no-priv", "auth-priv"]
    auth_proto: Literal["md5", "sha1", "sha224", "sha256", "sha384", "sha512"]
    auth_pwd: str
    priv_proto: Literal["aes128", "aes192", "aes192c", "aes256", "aes256c", "des"]
    priv_pwd: str


class ManagedSwitchRemotelogItem:
    """Nested item for remote-log field - supports attribute access."""
    name: str
    status: Literal["enable", "disable"]
    server: str
    port: int
    severity: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    csv: Literal["enable", "disable"]
    facility: Literal["kernel", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "ntp", "audit", "alert", "clock", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7"]


class ManagedSwitchMirrorItem:
    """Nested item for mirror field - supports attribute access."""
    name: str
    status: Literal["active", "inactive"]
    switching_packet: Literal["enable", "disable"]
    dst: str
    src_ingress: str
    src_egress: str


class ManagedSwitchStaticmacItem:
    """Nested item for static-mac field - supports attribute access."""
    id: int
    type: Literal["static", "sticky"]
    vlan: str
    mac: str
    interface: str
    description: str


class ManagedSwitchCustomcommandItem:
    """Nested item for custom-command field - supports attribute access."""
    command_entry: str
    command_name: str


class ManagedSwitchDhcpsnoopingstaticclientItem:
    """Nested item for dhcp-snooping-static-client field - supports attribute access."""
    name: str
    vlan: str
    ip: str
    mac: str
    port: str


class ManagedSwitchRoutervrfItem:
    """Nested item for router-vrf field - supports attribute access."""
    name: str
    switch_id: str
    vrfid: int


class ManagedSwitchSysteminterfaceItem:
    """Nested item for system-interface field - supports attribute access."""
    name: str
    switch_id: str
    mode: Literal["static", "dhcp"]
    ip: str
    status: Literal["disable", "enable"]
    allowaccess: Literal["ping", "https", "http", "ssh", "snmp", "telnet", "radius-acct"]
    vlan: str
    type: Literal["vlan", "physical"]
    interface: str
    vrf: str


class ManagedSwitchRouterstaticItem:
    """Nested item for router-static field - supports attribute access."""
    id: int
    switch_id: str
    blackhole: Literal["disable", "enable"]
    comment: str
    device: str
    distance: int
    dst: str
    dynamic_gateway: Literal["disable", "enable"]
    gateway: str
    status: Literal["disable", "enable"]
    vrf: str


class ManagedSwitchSystemdhcpserverItem:
    """Nested item for system-dhcp-server field - supports attribute access."""
    id: int
    switch_id: str
    status: Literal["disable", "enable"]
    lease_time: int
    dns_service: Literal["local", "default", "specify"]
    dns_server1: str
    dns_server2: str
    dns_server3: str
    ntp_service: Literal["local", "default", "specify"]
    ntp_server1: str
    ntp_server2: str
    ntp_server3: str
    default_gateway: str
    netmask: str
    interface: str
    ip_range: str
    options: str


class ManagedSwitchPayload(TypedDict, total=False):
    """Payload type for ManagedSwitch operations."""
    switch_id: str
    sn: str
    description: str
    switch_profile: str
    access_profile: str
    purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
    fsw_wan1_peer: str
    fsw_wan1_admin: Literal["discovered", "disable", "enable"]
    poe_pre_standard_detection: Literal["enable", "disable"]
    dhcp_server_access_list: Literal["global", "enable", "disable"]
    poe_detection_type: int
    max_poe_budget: int
    directly_connected: int
    version: int
    max_allowed_trunk_members: int
    pre_provisioned: int
    l3_discovered: int
    mgmt_mode: int
    tunnel_discovered: int
    tdr_supported: str
    dynamic_capability: str
    switch_device_tag: str
    switch_dhcp_opt43_key: str
    mclag_igmp_snooping_aware: Literal["enable", "disable"]
    dynamically_discovered: int
    ptp_status: Literal["disable", "enable"]
    ptp_profile: str
    radius_nas_ip_override: Literal["disable", "enable"]
    radius_nas_ip: str
    route_offload: Literal["disable", "enable"]
    route_offload_mclag: Literal["disable", "enable"]
    route_offload_router: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouteoffloadrouterItem]
    vlan: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchVlanItem]
    type: Literal["virtual", "physical"]
    owner_vdom: str
    flow_identity: str
    staged_image_version: str
    delayed_restart_trigger: int
    firmware_provision: Literal["enable", "disable"]
    firmware_provision_version: str
    firmware_provision_latest: Literal["disable", "once"]
    ports: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchPortsItem]
    ip_source_guard: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchIpsourceguardItem]
    stp_settings: str
    stp_instance: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStpinstanceItem]
    override_snmp_sysinfo: Literal["disable", "enable"]
    snmp_sysinfo: str
    override_snmp_trap_threshold: Literal["enable", "disable"]
    snmp_trap_threshold: str
    override_snmp_community: Literal["enable", "disable"]
    snmp_community: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpcommunityItem]
    override_snmp_user: Literal["enable", "disable"]
    snmp_user: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpuserItem]
    qos_drop_policy: Literal["taildrop", "random-early-detection"]
    qos_red_probability: int
    switch_log: str
    remote_log: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRemotelogItem]
    storm_control: str
    mirror: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchMirrorItem]
    static_mac: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStaticmacItem]
    custom_command: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchCustomcommandItem]
    dhcp_snooping_static_client: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchDhcpsnoopingstaticclientItem]
    igmp_snooping: str
    x802_1X_settings: str
    router_vrf: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRoutervrfItem]
    system_interface: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSysteminterfaceItem]
    router_static: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouterstaticItem]
    system_dhcp_server: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSystemdhcpserverItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ManagedSwitchResponse(TypedDict, total=False):
    """Response type for ManagedSwitch - use with .dict property for typed dict access."""
    switch_id: str
    sn: str
    description: str
    switch_profile: str
    access_profile: str
    purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
    fsw_wan1_peer: str
    fsw_wan1_admin: Literal["discovered", "disable", "enable"]
    poe_pre_standard_detection: Literal["enable", "disable"]
    dhcp_server_access_list: Literal["global", "enable", "disable"]
    poe_detection_type: int
    max_poe_budget: int
    directly_connected: int
    version: int
    max_allowed_trunk_members: int
    pre_provisioned: int
    l3_discovered: int
    mgmt_mode: int
    tunnel_discovered: int
    tdr_supported: str
    dynamic_capability: str
    switch_device_tag: str
    switch_dhcp_opt43_key: str
    mclag_igmp_snooping_aware: Literal["enable", "disable"]
    dynamically_discovered: int
    ptp_status: Literal["disable", "enable"]
    ptp_profile: str
    radius_nas_ip_override: Literal["disable", "enable"]
    radius_nas_ip: str
    route_offload: Literal["disable", "enable"]
    route_offload_mclag: Literal["disable", "enable"]
    route_offload_router: list[ManagedSwitchRouteoffloadrouterItem]
    vlan: list[ManagedSwitchVlanItem]
    type: Literal["virtual", "physical"]
    owner_vdom: str
    flow_identity: str
    staged_image_version: str
    delayed_restart_trigger: int
    firmware_provision: Literal["enable", "disable"]
    firmware_provision_version: str
    firmware_provision_latest: Literal["disable", "once"]
    ports: list[ManagedSwitchPortsItem]
    ip_source_guard: list[ManagedSwitchIpsourceguardItem]
    stp_settings: str
    stp_instance: list[ManagedSwitchStpinstanceItem]
    override_snmp_sysinfo: Literal["disable", "enable"]
    snmp_sysinfo: str
    override_snmp_trap_threshold: Literal["enable", "disable"]
    snmp_trap_threshold: str
    override_snmp_community: Literal["enable", "disable"]
    snmp_community: list[ManagedSwitchSnmpcommunityItem]
    override_snmp_user: Literal["enable", "disable"]
    snmp_user: list[ManagedSwitchSnmpuserItem]
    qos_drop_policy: Literal["taildrop", "random-early-detection"]
    qos_red_probability: int
    switch_log: str
    remote_log: list[ManagedSwitchRemotelogItem]
    storm_control: str
    mirror: list[ManagedSwitchMirrorItem]
    static_mac: list[ManagedSwitchStaticmacItem]
    custom_command: list[ManagedSwitchCustomcommandItem]
    dhcp_snooping_static_client: list[ManagedSwitchDhcpsnoopingstaticclientItem]
    igmp_snooping: str
    x802_1X_settings: str
    router_vrf: list[ManagedSwitchRoutervrfItem]
    system_interface: list[ManagedSwitchSysteminterfaceItem]
    router_static: list[ManagedSwitchRouterstaticItem]
    system_dhcp_server: list[ManagedSwitchSystemdhcpserverItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ManagedSwitchObject(FortiObject):
    """Typed FortiObject for ManagedSwitch with field access."""
    switch_id: str
    sn: str
    description: str
    switch_profile: str
    access_profile: str
    purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
    fsw_wan1_peer: str
    fsw_wan1_admin: Literal["discovered", "disable", "enable"]
    poe_pre_standard_detection: Literal["enable", "disable"]
    dhcp_server_access_list: Literal["global", "enable", "disable"]
    poe_detection_type: int
    max_poe_budget: int
    directly_connected: int
    max_allowed_trunk_members: int
    pre_provisioned: int
    l3_discovered: int
    mgmt_mode: int
    tunnel_discovered: int
    tdr_supported: str
    dynamic_capability: str
    switch_device_tag: str
    switch_dhcp_opt43_key: str
    mclag_igmp_snooping_aware: Literal["enable", "disable"]
    dynamically_discovered: int
    ptp_status: Literal["disable", "enable"]
    ptp_profile: str
    radius_nas_ip_override: Literal["disable", "enable"]
    radius_nas_ip: str
    route_offload: Literal["disable", "enable"]
    route_offload_mclag: Literal["disable", "enable"]
    route_offload_router: list[ManagedSwitchRouteoffloadrouterItem]
    vlan: list[ManagedSwitchVlanItem]
    type: Literal["virtual", "physical"]
    owner_vdom: str
    flow_identity: str
    staged_image_version: str
    delayed_restart_trigger: int
    firmware_provision: Literal["enable", "disable"]
    firmware_provision_version: str
    firmware_provision_latest: Literal["disable", "once"]
    ports: list[ManagedSwitchPortsItem]
    ip_source_guard: list[ManagedSwitchIpsourceguardItem]
    stp_settings: str
    stp_instance: list[ManagedSwitchStpinstanceItem]
    override_snmp_sysinfo: Literal["disable", "enable"]
    snmp_sysinfo: str
    override_snmp_trap_threshold: Literal["enable", "disable"]
    snmp_trap_threshold: str
    override_snmp_community: Literal["enable", "disable"]
    snmp_community: list[ManagedSwitchSnmpcommunityItem]
    override_snmp_user: Literal["enable", "disable"]
    snmp_user: list[ManagedSwitchSnmpuserItem]
    qos_drop_policy: Literal["taildrop", "random-early-detection"]
    qos_red_probability: int
    switch_log: str
    remote_log: list[ManagedSwitchRemotelogItem]
    storm_control: str
    mirror: list[ManagedSwitchMirrorItem]
    static_mac: list[ManagedSwitchStaticmacItem]
    custom_command: list[ManagedSwitchCustomcommandItem]
    dhcp_snooping_static_client: list[ManagedSwitchDhcpsnoopingstaticclientItem]
    igmp_snooping: str
    x802_1X_settings: str
    router_vrf: list[ManagedSwitchRoutervrfItem]
    system_interface: list[ManagedSwitchSysteminterfaceItem]
    router_static: list[ManagedSwitchRouterstaticItem]
    system_dhcp_server: list[ManagedSwitchSystemdhcpserverItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class ManagedSwitch:
    """
    
    Endpoint: switch_controller/managed_switch
    Category: cmdb
    MKey: switch-id
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        switch_id: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ManagedSwitchObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ManagedSwitchObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ManagedSwitchPayload | None = ...,
        switch_id: str | None = ...,
        sn: str | None = ...,
        description: str | None = ...,
        switch_profile: str | None = ...,
        access_profile: str | None = ...,
        purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"] | None = ...,
        fsw_wan1_peer: str | None = ...,
        fsw_wan1_admin: Literal["discovered", "disable", "enable"] | None = ...,
        poe_pre_standard_detection: Literal["enable", "disable"] | None = ...,
        dhcp_server_access_list: Literal["global", "enable", "disable"] | None = ...,
        poe_detection_type: int | None = ...,
        max_poe_budget: int | None = ...,
        directly_connected: int | None = ...,
        version: int | None = ...,
        max_allowed_trunk_members: int | None = ...,
        pre_provisioned: int | None = ...,
        l3_discovered: int | None = ...,
        mgmt_mode: int | None = ...,
        tunnel_discovered: int | None = ...,
        tdr_supported: str | None = ...,
        dynamic_capability: str | None = ...,
        switch_device_tag: str | None = ...,
        switch_dhcp_opt43_key: str | None = ...,
        mclag_igmp_snooping_aware: Literal["enable", "disable"] | None = ...,
        dynamically_discovered: int | None = ...,
        ptp_status: Literal["disable", "enable"] | None = ...,
        ptp_profile: str | None = ...,
        radius_nas_ip_override: Literal["disable", "enable"] | None = ...,
        radius_nas_ip: str | None = ...,
        route_offload: Literal["disable", "enable"] | None = ...,
        route_offload_mclag: Literal["disable", "enable"] | None = ...,
        route_offload_router: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouteoffloadrouterItem] | None = ...,
        vlan: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchVlanItem] | None = ...,
        type: Literal["virtual", "physical"] | None = ...,
        owner_vdom: str | None = ...,
        flow_identity: str | None = ...,
        staged_image_version: str | None = ...,
        delayed_restart_trigger: int | None = ...,
        firmware_provision: Literal["enable", "disable"] | None = ...,
        firmware_provision_version: str | None = ...,
        firmware_provision_latest: Literal["disable", "once"] | None = ...,
        ports: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchPortsItem] | None = ...,
        ip_source_guard: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchIpsourceguardItem] | None = ...,
        stp_settings: str | None = ...,
        stp_instance: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStpinstanceItem] | None = ...,
        override_snmp_sysinfo: Literal["disable", "enable"] | None = ...,
        snmp_sysinfo: str | None = ...,
        override_snmp_trap_threshold: Literal["enable", "disable"] | None = ...,
        snmp_trap_threshold: str | None = ...,
        override_snmp_community: Literal["enable", "disable"] | None = ...,
        snmp_community: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpcommunityItem] | None = ...,
        override_snmp_user: Literal["enable", "disable"] | None = ...,
        snmp_user: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpuserItem] | None = ...,
        qos_drop_policy: Literal["taildrop", "random-early-detection"] | None = ...,
        qos_red_probability: int | None = ...,
        switch_log: str | None = ...,
        remote_log: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRemotelogItem] | None = ...,
        storm_control: str | None = ...,
        mirror: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchMirrorItem] | None = ...,
        static_mac: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStaticmacItem] | None = ...,
        custom_command: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchCustomcommandItem] | None = ...,
        dhcp_snooping_static_client: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchDhcpsnoopingstaticclientItem] | None = ...,
        igmp_snooping: str | None = ...,
        x802_1X_settings: str | None = ...,
        router_vrf: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRoutervrfItem] | None = ...,
        system_interface: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSysteminterfaceItem] | None = ...,
        router_static: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouterstaticItem] | None = ...,
        system_dhcp_server: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSystemdhcpserverItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ManagedSwitchObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ManagedSwitchPayload | None = ...,
        switch_id: str | None = ...,
        sn: str | None = ...,
        description: str | None = ...,
        switch_profile: str | None = ...,
        access_profile: str | None = ...,
        purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"] | None = ...,
        fsw_wan1_peer: str | None = ...,
        fsw_wan1_admin: Literal["discovered", "disable", "enable"] | None = ...,
        poe_pre_standard_detection: Literal["enable", "disable"] | None = ...,
        dhcp_server_access_list: Literal["global", "enable", "disable"] | None = ...,
        poe_detection_type: int | None = ...,
        max_poe_budget: int | None = ...,
        directly_connected: int | None = ...,
        version: int | None = ...,
        max_allowed_trunk_members: int | None = ...,
        pre_provisioned: int | None = ...,
        l3_discovered: int | None = ...,
        mgmt_mode: int | None = ...,
        tunnel_discovered: int | None = ...,
        tdr_supported: str | None = ...,
        dynamic_capability: str | None = ...,
        switch_device_tag: str | None = ...,
        switch_dhcp_opt43_key: str | None = ...,
        mclag_igmp_snooping_aware: Literal["enable", "disable"] | None = ...,
        dynamically_discovered: int | None = ...,
        ptp_status: Literal["disable", "enable"] | None = ...,
        ptp_profile: str | None = ...,
        radius_nas_ip_override: Literal["disable", "enable"] | None = ...,
        radius_nas_ip: str | None = ...,
        route_offload: Literal["disable", "enable"] | None = ...,
        route_offload_mclag: Literal["disable", "enable"] | None = ...,
        route_offload_router: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouteoffloadrouterItem] | None = ...,
        vlan: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchVlanItem] | None = ...,
        type: Literal["virtual", "physical"] | None = ...,
        owner_vdom: str | None = ...,
        flow_identity: str | None = ...,
        staged_image_version: str | None = ...,
        delayed_restart_trigger: int | None = ...,
        firmware_provision: Literal["enable", "disable"] | None = ...,
        firmware_provision_version: str | None = ...,
        firmware_provision_latest: Literal["disable", "once"] | None = ...,
        ports: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchPortsItem] | None = ...,
        ip_source_guard: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchIpsourceguardItem] | None = ...,
        stp_settings: str | None = ...,
        stp_instance: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStpinstanceItem] | None = ...,
        override_snmp_sysinfo: Literal["disable", "enable"] | None = ...,
        snmp_sysinfo: str | None = ...,
        override_snmp_trap_threshold: Literal["enable", "disable"] | None = ...,
        snmp_trap_threshold: str | None = ...,
        override_snmp_community: Literal["enable", "disable"] | None = ...,
        snmp_community: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpcommunityItem] | None = ...,
        override_snmp_user: Literal["enable", "disable"] | None = ...,
        snmp_user: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpuserItem] | None = ...,
        qos_drop_policy: Literal["taildrop", "random-early-detection"] | None = ...,
        qos_red_probability: int | None = ...,
        switch_log: str | None = ...,
        remote_log: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRemotelogItem] | None = ...,
        storm_control: str | None = ...,
        mirror: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchMirrorItem] | None = ...,
        static_mac: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStaticmacItem] | None = ...,
        custom_command: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchCustomcommandItem] | None = ...,
        dhcp_snooping_static_client: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchDhcpsnoopingstaticclientItem] | None = ...,
        igmp_snooping: str | None = ...,
        x802_1X_settings: str | None = ...,
        router_vrf: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRoutervrfItem] | None = ...,
        system_interface: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSysteminterfaceItem] | None = ...,
        router_static: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouterstaticItem] | None = ...,
        system_dhcp_server: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSystemdhcpserverItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ManagedSwitchObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        switch_id: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        switch_id: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ManagedSwitchPayload | None = ...,
        switch_id: str | None = ...,
        sn: str | None = ...,
        description: str | None = ...,
        switch_profile: str | None = ...,
        access_profile: str | None = ...,
        purdue_level: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"] | None = ...,
        fsw_wan1_peer: str | None = ...,
        fsw_wan1_admin: Literal["discovered", "disable", "enable"] | None = ...,
        poe_pre_standard_detection: Literal["enable", "disable"] | None = ...,
        dhcp_server_access_list: Literal["global", "enable", "disable"] | None = ...,
        poe_detection_type: int | None = ...,
        max_poe_budget: int | None = ...,
        directly_connected: int | None = ...,
        version: int | None = ...,
        max_allowed_trunk_members: int | None = ...,
        pre_provisioned: int | None = ...,
        l3_discovered: int | None = ...,
        mgmt_mode: int | None = ...,
        tunnel_discovered: int | None = ...,
        tdr_supported: str | None = ...,
        dynamic_capability: str | None = ...,
        switch_device_tag: str | None = ...,
        switch_dhcp_opt43_key: str | None = ...,
        mclag_igmp_snooping_aware: Literal["enable", "disable"] | None = ...,
        dynamically_discovered: int | None = ...,
        ptp_status: Literal["disable", "enable"] | None = ...,
        ptp_profile: str | None = ...,
        radius_nas_ip_override: Literal["disable", "enable"] | None = ...,
        radius_nas_ip: str | None = ...,
        route_offload: Literal["disable", "enable"] | None = ...,
        route_offload_mclag: Literal["disable", "enable"] | None = ...,
        route_offload_router: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouteoffloadrouterItem] | None = ...,
        vlan: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchVlanItem] | None = ...,
        type: Literal["virtual", "physical"] | None = ...,
        owner_vdom: str | None = ...,
        flow_identity: str | None = ...,
        staged_image_version: str | None = ...,
        delayed_restart_trigger: int | None = ...,
        firmware_provision: Literal["enable", "disable"] | None = ...,
        firmware_provision_version: str | None = ...,
        firmware_provision_latest: Literal["disable", "once"] | None = ...,
        ports: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchPortsItem] | None = ...,
        ip_source_guard: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchIpsourceguardItem] | None = ...,
        stp_settings: str | None = ...,
        stp_instance: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStpinstanceItem] | None = ...,
        override_snmp_sysinfo: Literal["disable", "enable"] | None = ...,
        snmp_sysinfo: str | None = ...,
        override_snmp_trap_threshold: Literal["enable", "disable"] | None = ...,
        snmp_trap_threshold: str | None = ...,
        override_snmp_community: Literal["enable", "disable"] | None = ...,
        snmp_community: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpcommunityItem] | None = ...,
        override_snmp_user: Literal["enable", "disable"] | None = ...,
        snmp_user: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSnmpuserItem] | None = ...,
        qos_drop_policy: Literal["taildrop", "random-early-detection"] | None = ...,
        qos_red_probability: int | None = ...,
        switch_log: str | None = ...,
        remote_log: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRemotelogItem] | None = ...,
        storm_control: str | None = ...,
        mirror: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchMirrorItem] | None = ...,
        static_mac: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchStaticmacItem] | None = ...,
        custom_command: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchCustomcommandItem] | None = ...,
        dhcp_snooping_static_client: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchDhcpsnoopingstaticclientItem] | None = ...,
        igmp_snooping: str | None = ...,
        x802_1X_settings: str | None = ...,
        router_vrf: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRoutervrfItem] | None = ...,
        system_interface: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSysteminterfaceItem] | None = ...,
        router_static: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchRouterstaticItem] | None = ...,
        system_dhcp_server: str | list[str] | list[dict[str, Any]] | list[ManagedSwitchSystemdhcpserverItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "ManagedSwitch",
    "ManagedSwitchPayload",
    "ManagedSwitchResponse",
    "ManagedSwitchObject",
]